* ch02_all.do
* Sampling - all examples
*
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch02\data		/* this line is computer dependent */
	capture log close ch02_all
	log using ch02_all, name(ch02_all) replace 

*************************************************************************
* Ex 2.2 - 2.4 Basic calcuations
*************************************************************************
	use dairy_dis.dta, clear
	* compute correct sampling weights specific for neospora
		bysort herd: egen ts_neo=total(neospora!=.)
		gen wt_neo=1/(prob_hrd * (ts_neo/tot_cow))

	* compute prevalence - simple random sample
		svyset cow, clear
		svy: prop neospora
			matrix V = e(V)
			scalar var_srs=V[1,1]
	* computer prevalence - stratified sample
		svyset cow, strata(prov)
		svy: prop neospora
		svy: prop neospora, over(prov)
	* compute prevalence - stratified and sampling weights
		svyset, clear
		svyset cow [pweight=wt_neo], strata(prov)
		svy: prop neospora
	* compute prevalence - clustering only
		svyset, clear
		svyset herd 
		svy: prop neospora
			matrix V = e(V)
			scalar var_cl=V[1,1]
			di "design effect =  "var_cl/var_srs
	* compute prevalence - stratified, sampling weights and clustering
		svyset, clear
		svyset herd [pweight=wt_neo], strata(prov)
		svy: prop neospora

*************************************************************************
* Ex 2.5 - 2.6 Sample size of steers for vaccine trial
*************************************************************************
	sampsi .15 .1, p(.8) nocontinuity 	
	sampsi .15 .1, p(.8) 
	* Sample size with adjustment for clustering
		sampsi .15 .1, p(.8) nocontinuity 	
		sampclus, rho(0.3) obs(50) 	

*************************************************************************
* Ex 2.7 Power Calculation by Simulation
*************************************************************************
	set seed 123456
	use pig_adg.dta, clear
	* generate a new variable for the presence/absence of worms
		gen w2=worms>0
	* regress adg on w2, sex and farm
		xi:regress adg w2 sex i.farm
		* compute predicted values
			predict pred, xb
		* revised the predicted values to reflect an effect of 15 gm/day gain in adg
			replace pred=pred + (w2*7.7) - (w2*15)
	* set up a file to hold the results from the simulation
		tempname memhold
		postfile `memhold' beta z pval using ss_sim_rslt.dta, replace
	* set up loop to repeat the analyses many times
		local i=0
		while `i'<10000 {
		* generate -adg_new- normally distributed with mean = pred. val. and SD=46.905
			gen rand=invnorm(uniform())
			gen adg_new=pred+46.905*rand
		* regress adg_new on the same set of predictors
			quietly xi:regress adg_new w2 sex i.farm
  			* obtain the regression coefficient and compute its z statistic and P-value
			matrix V=e(V)
			matrix B=e(b)
			scalar beta=B[1,1]
			scalar z=B[1,1]/sqrt(V[1,1])
			scalar pval=2*(1-normprob(abs(z)))
   		* post the beta, z and pval to the results file
			post `memhold' (beta) (z) (pval)
   	* repeat the loop
		drop rand adg_new
		local i = `i'+1
		}
	* stop the process of posting results to a file
		postclose `memhold'
	* open the file that captured the results
		use ss_sim_rslt.dta, clear
   	* compute and display the power
		count if pval<0.05
		scalar power=r(N)/10000
		display "Power is " %8.3f scalar(power)
   	* compare the computed power to a calculated value
		sampsi 500 515, sd(46.905) n1(114) n2(227)


