* ch5_all.do
* Screening and Diagnostic Tests - all calculations
*
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch05\data		/* this line is computer dependent */
	capture log close ch05_all
	log using ch05_all, name(ch05_all) replace 

*************************************************************************
* Ex 5.1 Measuring precision and agreement - quantitative test results
*************************************************************************

* open the elisa_repeat dataset
	use elisa_repeat.dta, clear
* CV
* compute the mean, standard deviations and CV of the 6 raw and 6 adjusted values
	egen raw_avg=rowmean(raw1-raw6)
	egen raw_sd=rowsd(raw1-raw6)
	gen raw_cv=raw_sd/raw_avg
	egen adj_avg=rowmean(adj1-adj6)
	egen adj_sd=rowsd(adj1-adj6)
	gen adj_cv=adj_sd/adj_avg
   	* compute the average CV for the raw and adjusted values
		sum raw_cv adj_cv
* Pearson correlation coefficient
	corr raw1 raw2 adj1 adj2
* Concordance correlation and plot
	concord raw1 raw2
	format adj1 adj2 %6.2f
	concord adj1 adj2, ccc(scheme(s1mono) lpattern(solid dash) ///
		ytitle("Adjusted OD - Sample 1", m(r+2)) ylabel(0.0(0.2)1.0, angle(horizontal)) ///
		xtitle("Adjusted OD - Sample 2", m(t+2)))
	graph save "figs/fig05_ccc.gph", replace
	graph export "figs/fig05_ccc.wmf", replace
* Limits of Agreement plot
	concord adj1 adj2, loa (scheme(s1mono) ytitle("difference between values", m(r+2)) ///
		xtitle("mean value", m(t+2)) legend(off) caption("") ///
		text(.1 .3 "observed average difference" -.15 .3 "lower limit of agreement" ///
		.31 .3 "upper limit of agreement") ylabel(-0.2(0.2)0.4, angle(horizontal))) 
	graph save "figs/fig05_loa.gph", replace
	graph export "figs/fig05_loa.wmf", replace

*************************************************************************
* Ex 5.2 Computing kappa to compare two tests with dichotomous outcome
*************************************************************************

* open the isa-test dataset
	use isa_test.dta, clear
* compute McNemar's chi-square for assessing the positive proportions
	mcc ifat1_np ifat2_np
* compute kappa using the dichotomized IFAT results (2 labs)
	kap ifat1_np ifat2_np
	* Note SE of kappa which does not assume independence of tests computed in SAS
* compute the confidence interval for kappa
	kapci ifat1_np ifat2_np
	* compare with bootstrap estimates
	kapci ifat1_np ifat2_np, est(bsall) reps(1000) 	/* results not shown in text */


*************************************************************************
* Ex 5.3 Agreement among ordinal test results
*************************************************************************

* open the isa-test dataset
	use isa_test.dta, clear
* compute an unweighted kappa
	kap ifat1 ifat2, tab
	kapci ifat1 ifat2, reps(1000) 
* compute the weighted kappa and confidence interval
	* set up the agreement (weight) matrix
		kapwgt isa_wt 1 \ 0.7 1 \ 0.3 0.7 1 \ 0 0.3 0.7 1 \ 0 0 0.3 0.7 1
	kap ifat1 ifat2, wgt(isa_wt) tab
	kapci ifat1 ifat2, reps(1000) wgt(isa_wt)

*************************************************************************
* Ex 5.4 Computing Se and Sp 
*************************************************************************

* Open the BVD_test dataset
	use bvd_test.dta, clear
	egen cp_92=cut(od), at(0 0.92 5) icodes
	diagt calfst cp_92 if spec==1		/* analysis limited to blood samples */


*************************************************************************
* Fig 5.4 Generating a figure for selecting a cutpoint
*************************************************************************

   twoway (function normalden(x,20,5), range(0 40)) (function normalden(x,32,3), range(20 44)), ///
      yline(0, lcolor(gs13)) ylabel(0(0.05)0.15, angle(horizontal)) ///
	xline(28) xlabel(0(10)50, ticks) scheme(s1mono) plotregion(lcolor(none)) ///
      text(.04 5 "false" "negatives", place(NE)) text(.07 40 "false" "positives", place(NE))   ///
	text(.1 27 "cutpoint", orientation(vertical))  legend(off)
	* add arrows using Stata graphics editor
	graph save "figs/fig05_cutpoint.gph", replace
	graph export "figs/fig05_cutpoint.wmf", replace

*************************************************************************
* Fig 5.5 Generating a Se - Sp plot (2 graph ROC)
*************************************************************************

* Open the BVD_test dataset
	use bvd_test.dta, clear
* generate the data	
	/* at the time of writing - roctg2 was not widely avaialable
	but can be obtained from the author Michael Reichenheim - michael@ims.uerj.br */
	roctg2 calfst od, cont nograph saving("figs/fig05_sesp_plot") replace
* generate the plot
	use "figs/fig05_sesp_plot", clear
	twoway (line sens score, sort lcolor(black) lpat(solid)) ///
		(rarea sens_l sens_u score, sort lcolor(black) lpat(dash) fcolor(none)) ///
		(line spec score, sort lcolor(black) lpat(solid)) ///
		(rarea spec_l spec_u score, sort lcolor(black) lpat(dash) fcolor(none)), ///
		ytitle("sensitivity / specificity (%)") ytitle(, margin(small)) ylab(0(20)100, angle(horizontal)) ///
		xtitle(cutpoint OD) xtitle(, margin(small)) ///
		legend(off) scheme(s1mono) text(90 1.7 "specificity", place(NE)) ///
		text(20 1.6 "sensitivity", place(NE)) 
	graph save "figs/fig05_sesp_plot.gph", replace
	graph export "figs/fig05_sesp_plot.wmf", replace

*************************************************************************
* Fig 5.6 ROC curves
*************************************************************************

* open the bvd-test dataset
	use bvd_test.dta, clear
* fit a parametric ROC curve
	rocfit calfst od, continuous(20)
* generate a plot of both non-parametric and parametric ROC curves
	rocplot , confband scheme(s1mono) plotopts(msymbol(o) msize(vsmall) mcolor(black)) ///
		lineopts(lcolor(black) lpat(dash)) ciopts(fcolor(none) lcolor(gs5)) ///
		ytitle(, m(r+2)) ylab(0(.20)1, angle(horizontal)) xtitle(, m(t+2)) ///
		note("Area under curve = 0.7075,    SE(area)=0.0165")
	graph save "figs/fig05_roc.gph", replace
	graph export "figs/fig05_roc.wmf", replace

*************************************************************************
* Ex 5.7 Likelihood ratios
*************************************************************************

* open the bvd-test dataset
	use bvd_test.dta, clear
* Cutpoint specific LR
	* compute Se and Sp at various cutpoints
	diagt calfst co_5
	diagt calfst co_7
	diagt calfst co_9
	diagt calfst co_11
	diagt calfst co_13
	diagt calfst co_15
	diagt calfst co_17
	diagt calfst co_19
* Category specific LR (categories using cutpoints as left ends of ranges)
* obtain the categories used for cutpoints
	egen od_cat=cut(od), at(0 .5 .7 .9 1.1 1.3 1.5 1.7 1.9 4)
* derive likelihoods
	tab od_cat calfst , col
		* divide col% for PI+ by col% for PI- to get LR


*************************************************************************
* Ex 5.8 Parallel and serial interpretation
*************************************************************************

* Open the isa_test dataset
	use isa_test.dta, clear
* cross-tabulate IFAT and PCR by VI
	bysort dz:tab ifat1_np pcr, row col
* generate results for series interpretation
	gen series = (ifat1 & pcr) if !missing(ifat1, pcr)
		tab dz series, row
* generate results for parallel interpretation
	gen parallel = ifat1 | pcr if !missing(ifat1, pcr)
		tab dz parallel, row


*************************************************************************
* Ex 5.11 Evaluations based on the composite Reference Standard
*************************************************************************

	use isa_lcm.dta, clear
* evaluating ifat with pcr2 as a reference test and vi as the resolver
	* compute the CRS psuedo-gold standard (pcr2=reference vi=resolver)
		gen crs = (pcr_b | vi_a) if !missing(pcr_b, vi_a)
	* evaluate ifat
		diagt crs ifat_c2


*************************************************************************
* Ex 5.12 Generating the tabular data required for TAGS
*************************************************************************

	use isa_lcm.dta, clear
* generate data for 3 tests (ifat, pcr_a, vi_a), 3 populations (low, med and high)
	* these data not used in text
	table ifat_c2 pcr_a vi_a if pop==1
	table ifat_c2 pcr_a vi_a if pop==2
	table ifat_c2 pcr_a vi_a if pop==3


*************************************************************************
* Ex 5.13 Evaluation of the effects of population of origin
*************************************************************************

	use isa_lcm.dta, clear
	* compute the CRS psuedo-gold standard (pcr2=reference vi=resolver)
		gen crs = (pcr_b | vi_a) if !missing(pcr_b, vi_a)
* logistic regression analysis of IFAT
	* unconditional estimate of Se and Sp
		logit ifat_c2 if crs==1
		logit ifat_c2 if crs==0
	* evaluate the effect of population on Se and Sp of IFAT
		keep if pop>0
		xi:logit ifat_c2 i.pop if crs==1
			test _Ipop_2 _Ipop_3 
		xi:logit ifat_c2 i.pop if crs==0
			test _Ipop_2 _Ipop_3 
	* evaluate effect of clustering on Se and Sp of IFAT
		xi:xtlogit ifat i.pop if crs==1, i(cage)
		xi:xtlogit ifat i.pop if crs==0, i(cage)

log close ch05_all
