* ch13_all.do
* Confoudning: detection and control - all calculations
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch13\data		/* this line is computer dependent */
	capture log close ch13_all
	log using ch13_all, name(ch13_all) replace 

*************************************************************************
* Disclaimer
*************************************************************************
 
* The analyses shown in this program are carried out solely for providing examples for the text. 

*************************************************************************
* Ex 13.7 Mantel-Haenszel - little or no confounding
*************************************************************************
use "C:\ver2\data\feedlot.dta", clear
	* create crude association table of BRD with MHSC
	* use epitab to create tables and measures of association
	cc brd mhcysc
	* create the tables by level of IBR
	bysort ibrsc: tab brd mhcysc, row
	* create the adjusted MH OR
	cc brd mhcysc, by(ibrsc)

*************************************************************************
* Ex 13.8 Mantel-Haenszel - confounding present
*************************************************************************
use "C:\ver2\data\feedlot.dta", clear
	* create the tables by level of Province
	bysort province: tab brd mhcysc, row
	* create the adjusted MH OR
	cc brd mhcysc, by(province)

*************************************************************************
* Ex 13.10 Mantel-Haenszel - interaction
*************************************************************************
use "C:\ver2\data\nocardia.dta", clear
	* create the tables by level of cloxacillin
	bysort dclox: tab casecont dneo, row
	* create the adjusted MH OR
	cc casecont dneo, by(dclox)

*************************************************************************
* Section 13.7.1 Open the first teaching data set from chapter 1 
*************************************************************************
* create files specifically for indirect standardization
use vacc_factual.dta, clear
	* create and save Table 13.4
	collapse (count) pop=subject (sum) cases=d, by(c e)
	bysort e: egen totcases=total(cases)
	save ind_vacc_summ.dta, replace

*************************************************************************
* Section 13.7.1 Standardize the rates
*************************************************************************
	use vacc_factual.dta, clear
	* create the std pop rates based on non-exposed subjects
	tab c d if e==0, row
	*std pop file called "ind_stdpop"
	* create summary from vacc_factual
	bysort c: tab e d, row
	* summary file called ind_vacc_summ 
	use ind_vacc_summ, clear
	istdize totcases pop c using ind_stdpop, by(e) rate(jth_rate crude_rate)
	* the adjusted (standardized) rates are equal

*************************************************************************
* Section 13.7.2 Create marginal structural model based on total population
*************************************************************************
	use vacc_factual.dta, clear
	collapse (count) count=subject, by(c e d)
	* create probability of exposure by confounder level
	logit e c [fw=count]
	predict p, p
	* create probability of non-exposure in non-exposed
	replace p=1-p if e==0
	* gen total weight and then create total psuedo-population
	gen wtot=1/p
	gen pseudopop_tot=count*wtot
		* perform analysis
		tab e d [fw= pseudopop_tot], col
		cs d e [fw= pseudopop_tot]
		* proporions of disease by exposure are equal
		* RR = 1

*************************************************************************
* Section 13.7.2 Create MS model based on exposed as the standard
*************************************************************************
	use vacc_factual.dta, clear
	collapse (count) count=subject, by(c e d)
	* create probability of exposure by confounder level
	logit e c [fw=count]
	predict p, p
	* gen exposed and unexposed weights and then create exposed psuedo-population
	gen wexp=1 if e==1
	replace wexp=p/(1-p) if e==0
	gen pseudopop_exp=count*wexp
		* perform analysis
		tab e d [fw= pseudopop_exp], col
		cs d e [fw= pseudopop_exp]
		* proporions of disease by exposure are equal
		* RR = 1

*************************************************************************
* Propensity scores
*************************************************************************
use "C:\ver2\data\feedlot.dta", clear
	drop if brsvsc==. | hssc==.
*************************************************************************
* computing propensity scores
*************************************************************************
	* first compute unconditional association between -brd- and -mhcysc- 
		cs brd mhcysc
	* compute propensity scores (based on -brsvsc-, -hssc-, prov and wt0
	  	xi:pscore mhcysc brsvsc hssc i.prov wt0, pscore(ps) logit comsup ///
			blockid(block) numblo(10)
		tab block, sum(ps)
	* recompute unconditional assoc. using obs. with common support
		cs brd mhcysc if comsup==1
*************************************************************************
* Ex 13.4 matching using propensity scores
*************************************************************************
	* nearest neighbour matching restricted to area of common support
		attnd brd mhcysc ,  comsup pscore(ps) detail
			* appears to reduce effect of -mycysc- (att=0.072)
	* radius matching based on pscore in area of common support
		attr brd mhcysc , comsup pscore(ps) radius(0.01)
			* appears to increase effect of -mycysc- (att=0.143)
*************************************************************************
* Ex 13.11 using pscore in a stratified analysis
*************************************************************************
	atts brd mhcysc, pscore(ps) blockid(block) comsup detail
	* appears to increase effect of -mycysc- (att=0.144)
*************************************************************************
* Ex 13.12 using pscore in a logistic regression model
*************************************************************************
	* unconditional OR
		logit brd mhcysc if comsup, or				/* OR = 1.73, p=0.019 */
	* OR adjusted for ps - either continuous or categorical
		logit brd mhcysc ps if comsup, or				/* OR = 2.24, p=0.001 */
		xi:logit brd mhcysc i.block if comsup, or	/* OR = 2.39, p=0.000 */
	* comparison with direct control of confounders
		  xi:logit brd mhcysc brsvsc hssc i.prov wt0 if comsup, or 	/* OR = 2.25, p=0.001 */

