* ch14_all.do
* Linear regression - all calculations
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch14\data		/* this line is computer dependent */
	capture log close ch14_all
	log using ch14_all, name(ch14_all) replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the daisy2 dataset
Note: The issue of cows clustering within herds is ignored in these analyses
	Clustering is dealt with in Chapters 20-23
*/

*************************************************************************
* Open the data and restrict the observations saved
*************************************************************************

use daisy2.dta, clear
	keep if wpc~=.	/* keep only records with valid wait period to conception intervals */
	keep if h7==1	/* keep only records from the 7 herds with high disease rates */
	* kept, but ignored, cows with multiple lactations

*************************************************************************
* Ex 14.1 - 14.2 Simple linear regression models - 120 day milk production as outcome
*************************************************************************
	* continuous predictor
		summ milk120, d
		regress milk120 parity
		predict pv, xb 
		predict stdp, stdp  /*SE of population prediction */
		predict stdf, stdf  /*SE of individual prediction */
		drop pv
	* prediction intervals
		twoway (scatter milk120 parity) ///
			(lfitci milk120 parity, ciplot(rline) blcolor(black) blpattern(dash)) ///
			(lfitci milk120 parity, stdf ciplot(rline) blcolor(black) blwidth(medium) ///
				blpattern(shortdash_dot_dot)), ///
			legend(off) scheme(s1mono) ytitle("120 day milk production (l)", m(r+2)) /// 
			ylabel(, angle(horizontal) format(%4.0f)) xtitle("parity", m(t+2)) ///
			plotregion(style(none)) 
	graph save "figs/fig14_predict.gph", replace
	graph export "figs/fig14_predict.wmf", replace
	* dichotomous predictor
		regress milk120 dyst

*************************************************************************
* Ex 14.3 Assessing the overall significance of multiple predictors
*************************************************************************
	xi: reg milk120 parity dyst i.herd
	xi: reg milk120 parity 
	xi: reg milk120 parity twin dyst rp vag_disch
		test twin dyst rp vag_disch

*************************************************************************
* Ex 14.4 Modifying (scaling) the predictors
*************************************************************************
	* scaling parity
		gen parity1=parity-1
		reg milk120 parity1
	* scaling milk120
		gen milk120k=milk120/1000
	* scaling herd_size
		gen hs100=herd_size/100
		summ hs100, d

*************************************************************************
* Ex 14.6 - 14.7 Coding indicator variables
*************************************************************************
	* coding indicator variables
		xi: reg milk120  parity1 i.herd
	* hierarchical coding
		xi: reg milk120 i.parity1	/* indicator coding */
			estimates store indic
		xi3: reg milk120 b.parity1	/* hierarchical coding */
			estimates store hier
		estimates table indic hier

*************************************************************************
* Ex 14.8 Modelling correlated predictors - VIF
*************************************************************************
	* quadratic regression without centring
		reg wpc hs100
		gen hs100_sq=hs100^2 /* creating a squared scaled term */
		corr hs100 hs100_sq
		reg wpc hs100 hs100_sq
		vif
	* quadratic regression with centring
		gen hs100_ct = hs100-2.51		/* using centred scaled values of herd_size */
		gen hs100_ctsq = hs100_ct^2
		corr hs100_ct hs100_ctsq
		reg wpc hs100_ct hs100_ctsq
		vif
	
*************************************************************************
* Ex 14.9 - 14.11 Interactions
*************************************************************************
	* two dihcotomous predictors
		reg wpc rp vag_disch
		xi: reg wpc i.rp*vag_disch
	* a dichotomous and a continuous predictor
		reg wpc dyst milk120k
		xi: reg wpc i.dyst*milk120k
		* graph the predicted values
			predict pv, xb
			sort milk120k
			twoway (line pv milk120k if dyst==0, lcol(black) lpat(dash)) ///
				(line pv milk120k if dyst==1, lcol(black) lpat(solid)) ///
				(scatter wpc milk120k, ms(o) msize(tiny)), ///
				xtitle("120 day milk production ('000 kgs)", m(t+2)) ///
				ytitle("wait period to conception interval", m(r+2)) ylabel(, angle(horizontal)) ///
				legend(off) scheme(s1mono) plotregion(style(none)) ///
				text(120 4.5 "dystocia", place(NE)) text(40 5.2 "no dystocia", place(NE))
			graph save "figs/fig14_interact.gph", replace
			graph export "figs/fig14_interact.wmf", replace
		drop pv
	* two continuous predictors
		gen p_m=parity*milk120k
		xi: reg wpc parity milk120k p_m	/* interaction not significant, but plot for demonstration purposes */
		* graph the predicted values
			predict pv, xb
			egen parity_c3=cut(parity), at(1 2 3 4 99) 
			sort milk120k
			twoway (line pv milk120k if parity_c3==1 & wpc>=50 & wpc<=150, lcol(black) lpat(solid)) ///
				(line pv milk120k if parity_c3==2 & wpc>=50 & wpc<=150, lcol(black) lpat(dash)) ///
				(line pv milk120k if parity_c3==3 & wpc>=50 & wpc<=150, lcol(black) lpat(shortdash_dot)) ///
				(scatter wpc milk120k if wpc>=50 & wpc<=150, ms(o) msize(tiny)), ///
				xtitle("120 day milk production ('000 kgs)", m(t+2)) ///
				ytitle("wait period to conception interval", m(r+2)) ylabel(, angle(horizontal)) ///
				legend(off) scheme(s1mono) plotregion(style(none)) ///
				text(58 3.9 "1", place(NE)) text(58 4.7 "2", place(NE)) text(62 5.3 "3+", place(NE))
			graph save "figs/fig14_interact2.gph", replace
			graph export "figs/fig14_interact2.wmf", replace
		drop pv p_m parity_c3

*************************************************************************
* Ex 14.12 Causal model and multivariable model for wpc
*************************************************************************
	* generate a variable for calving between Feb and Jul
		gen calv_mth=month(calv_dt)
		gen aut_calv=(calv_mth>=2 & calv_mth<=7) if !missing(calv_mth)
	* fit a multivariable model
		xi: reg wpc aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
			lincom _Irp_1 + vag_disch + _IrpXvag_d_1

*************************************************************************
* Ex 14.13 - 14.14 Evaluation of model
*************************************************************************
	* refit the model
		xi: reg wpc aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
	* compute residuals and diagnostic statistics
		predict pv, xb
		predict rsta, rsta
		predict rstu, rstu
	* evaluation of predictive ability
		* r2 only 7%
	* evaluating homscedasticity
		rvfplot, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) msize(small)
		graph save "figs/fig14_het.gph", replace
		graph export "figs/fig14_het.wmf", replace
		estat hettest
		* significant heteroscedasticity - but not very apparent in plot
		* evaluate variation at different levels of -pv-
			egen pv_c4=cut(pv), at(40 60 80 100 999)
			tab pv_c4, sum(rsta)
			* appears to be due to lower variation at low and high values of pv
	* evaluating normality
		sum rsta rstu /* little difference between standardized and studentized */
		histogram rsta, freq bin(20) norm xtitle(, size(large) m(t+2)) ytitle(, size(large) m(r+2)) ///
			ylabel(, labsize(large) angle(horizontal)) xlabel(, labsize(large)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo, replace)
		qnorm rsta, xtitle(, size(large) m(t+2)) ytitle(, size(large) m(r+2)) ylabel(, labsize(large) angle(horizontal)) ///
			xlabel(, labsize(large))  scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm, replace)
		* combine the two graphs)
		graph combine fig14_norm_histo fig14_norm_qnorm, ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig14_norm.gph", replace
			graph export "figs/fig14_norm.wmf", replace
		swilk rsta
		* significant non-normality - right skewed
	drop pv rsta rstu 

*************************************************************************
* Ex 14.15 Visual evaluation of linearity 
*************************************************************************
xi: reg wpc aut_calv hs100_ct parity1 twin dyst i.rp*vag_disch
		predict pv, xb
		predict rsta, rsta
twoway (scatter rsta hs100_ct) (lowess rsta hs100_ct), scheme(s1mono) legend(off) ///
		ytitle("standardized residuals", m(r+2)) ylabel(-2 0 2 4, angle(horizontal) format(%4.2f)) ///
		xlabel(-1.5(.5)1, format(%4.2f)) xtitle("herd size (scaled and centred)", m(t+2)) plotregion(style(none))	
	graph save "figs/fig14_linear.gph", replace
	graph export "figs/fig14_linear.wmf", replace
	drop pv rsta

*************************************************************************
* Section 14.9.5 Dealing with issue of non-normality of resdiuals by using robust SE
*************************************************************************
	xi: reg wpc aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
		estimates store orig
	xi: reg wpc aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch, robust
		estimates store robust
	estimates table orig robust, p
	* -dyst- becomes non-significant but not much change in other P-values
	
*************************************************************************
* Section 14.9.6 Transformations
*************************************************************************
	* run the Box Cox procedure
	xi: boxcox wpc aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
		* theta = 0.11     perhaps a log transformation will be adequate
* try a log transformation 
		gen wpc_ln= ln(wpc)
		xi: reg wpc_ln aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
			* significance of predictors is roughly the same as previous model
	* compute residuals and diagnostic statistics
		predict pv, xb
		predict rsta, rsta
	* evaluation of predictive ability
		* r2 now 9.4%
	* evaluating homscedasticity
		rvfplot, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) msize(small)
		graph save "figs/fig14_het_ln.gph", replace
		graph export "figs/fig14_het_ln.wmf", replace
		estat hettest
		* heteroscedasticity is now much worse - completely unacceptable
	* evaluating normality
		histogram rsta, bin(20) norm xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo_ln, replace)
		qnorm rsta, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm_ln, replace)
		* combine the two graphs
		graph combine fig14_norm_histo_ln fig14_norm_qnorm_ln, ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig14_norm_ln.gph", replace
			graph export "figs/fig14_norm_ln.wmf", replace
		swilk rsta
		* non-normality is still significant, but visually the distribution is much better
		* a few very large negative residuals - cows with wpc<5 days (become very small values on log scale)
	drop pv rsta 

* try a square root transformation 
		gen wpc_sqrt= wpc^.5
		xi: reg wpc_sqrt aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
			* significance of predictors is roughly the same as previous model
	* compute residuals and diagnostic statistics
		predict pv, xb
		predict rsta, rsta
	* evaluation of predictive ability
		* r2 now 8.5%
	* evaluating homscedasticity
		rvfplot, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) msize(small)
		graph save "figs/fig14_het_sqrt.gph", replace
		graph export "figs/fig14_het_sqrt.wmf", replace
		estat hettest
		* heteroscedasticity is no longer significant
	* evaluating normality
		histogram rsta, bin(20) norm xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo_sqrt, replace)
		qnorm rsta, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm_sqrt, replace)
		* combine the two graphs
		graph combine fig14_norm_histo_sqrt fig14_norm_qnorm_sqrt, ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig14_norm_sqrt.gph", replace
			graph export "figs/fig14_norm_sqrt.wmf", replace
		swilk rsta
		* non-normality is still significant, but visually the distribution is somewhat better
* conclusion - use the square root transformed model given that homoscedasticity is more important than normality
	drop pv rsta

*************************************************************************
* Ex 14.16 Square root transformed model
*************************************************************************
* refit the model
	xi: reg wpc_sqrt aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch, robust
	predict pv, xb
	gen pv_sq=pv^2
	table rp vag_disch dyst, contents(mean pv_sq)
	drop pv pv_sq

*************************************************************************
* Ex 14.17 - 14.19 Evaluation of outliers and influential points
*************************************************************************
* refit the model with usual SEs
	xi: reg wpc_sqrt aut_calv hs100_ct hs100_ctsq parity1 twin dyst i.rp*vag_disch
		predict pv, xb
		predict rsta, rsta
		predict rstu, rstu
		predict lev, lev
		predict cooksd, cooksd
		predict dfit, dfit
		predict dfb_dyst, dfbeta(dyst)
		predict dfb_rp, dfbeta(_Irp_1)
		predict dfb_vd, dfbeta(vag_disch)
		predict dfb_int, dfbeta(_IrpXvag_d_1)
	* examine outliers
		sum rsta wpc wpc_sqrt pv
			* individual outliers based on t-statistic with 1564 df and sig at alpha/n
				di 2*1574*ttail(1564, 4.17)	/* P=0.05 */
				display invttail(1564,.025/1574)
			* cow with residuals > 4.17 or <-4.17 are outliers at two-tailed alpha =0.05 in a dataset of this size (there are none)
			* max rstu was 3.28
		* number of standardized residuals outside -2/+2 and -3/+3
			* expect 5% (n=72) outside -2,+2 and 1% (n=14) outside -3,+3
				count if rsta<-2 | rsta>2		/* n=73 */
				count if rsta<-3 | rsta>3		/* n=3 */
			* fewer very extreme residual values than expected
		format wpc_sqrt pv rsta %5.2f
		format herd_size parity1 %4.0f
		format twin dyst rp vag_disch %3.0f
		sort rsta
		list cow herd_size parity twin dyst rp vag_disch wpc_sqrt pv rsta in 1/5, clean noobs ab(5)
			* generally cows that were predicted to have short intervals ... but had very short intervals
		list cow herd_size parity twin dyst rp vag_disch wpc_sqrt pv rsta in -5/l, clean noobs ab(5)
			* generally cows that were predicted to have short intervals ... but had very long intervals
		* no obvious reason for concern
	* examine leverage
		sum lev cooksd dfit
		format lev cooksd dfit %5.3f
		* large leverage values (threshold is 3k/n = 3*9/1574 = 0.017
			count if lev>=0.017
				* many (n=119) high leverage values
			sort lev
			list cow herd_size parity1 twin dyst rp vag_disch rsta lev in -20/l, clean noobs ab(5)
				* cows with twins and/or reproductive diseases had high leverage
	* examine influence
		* Cooks Distance values (threshold is 4/n = 0.0025)
			count if cooksd>=0.0025
				* many (n=91) influential observations (based on Cooks D)
			sort cooksd
			list cow herd_size parity1 twin dyst rp vag_disch rsta cooksd in -20/l, clean noobs ab(5)
				* cows with twins and/or reproductive diseases were influential .... but
					* no reason to delete them
		* DFITS values (threshold = +/- 2*sqrt(k/n) = 0.15
			count if dfit>=0.15
				* many (n=68) influential observations (based on dfits)
			sort dfit
			list cow herd_size twin dyst rp vag_disch rsta cooksd dfit in 1/5,clean noobs ab(5)
			list cow herd_size twin dyst rp vag_disch rsta cooksd dfit in -5/l,clean noobs ab(5)
				* cows with twins and/or reproductive diseases were influential .... but
		* delta betas (threshold = 2/sqrt(n) = 0.050
			sort dfb_dyst
				count if dfb_dyst>0.050	/* n=28 */
				list cow herd_size parity1 twin dyst rp vag_disch rsta dfb_dyst in -10/l, clean noobs ab(5)
			sort dfb_rp
				count if dfb_rp>0.050	/* n=34 */
				list cow herd_size parity1 twin dyst rp vag_disch rsta dfb_rp in -10/l, clean noobs ab(5)
			sort dfb_vd
				count if dfb_vd>0.050	/* n=21 */
				list cow herd_size parity1 twin dyst rp vag_disch rsta dfb_vd in -10/l, clean noobs ab(5)
			sort dfb_int
				list cow herd_size parity1 twin dyst rp vag_disch rsta dfb_int in -10/l, clean noobs ab(5)
				count if dfb_int>0.050	/* n=34 */
			* not surprisingly, the cows with the diseases were influential for the respective coefficient
			* no reason to remove these observations ... 
			
