* ch15_all.do
* Model Building - all examples
*
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch15\data		/* this line is computer dependent */
	capture log close ch15_all
	log using ch15_all, name(ch15_all) replace 

*************************************************************************
* Ex 15.1 Cronbach's Alpha
*************************************************************************
	use pig_farm.dta, clear
		alpha exhaust inlet maninlt, std item
		alpha strdnst grwdnst fnrdnst shipm2, std item

*************************************************************************
* Ex 15.2 Correspondence Analysis   
*************************************************************************
	use tac_mca.dta, clear
	* carry out a multiple correspondence analysis of 6 variables
		mca  tac  X1 X2 X3 X4 X5 X6, method(joint) norm(principal)
		mcaplot (tac, msymb(S)) (X1, msymb(x)) (X2, msymb(Oh)) (X3, msymb(Dh)) ///
			(X4, msymb(Th)) (X5, msymb(s)) (X6, msymb(t)) , overlay origin legend(off) ///
			mlabangle(45) scheme(s1mono) note("") xlab(-.8(.2).4) ///
			mcol(black) ylabel(#2, angle(horizontal)) ylabel(#5)
		graph save figs/fig15_mca.gph, replace
		graph export figs/fig15_mca.wmf, replace

*************************************************************************
* Section 15.6 Evaluating the linearity of the effect of milk120 on calving to first service interval
*************************************************************************
	use daisy2.dta, clear
	* scatterplot
		* determining 2.5 and 97.5 percentiles of -milk120-
			centile  milk120, c(2.5 97.5)
				local m120_2_5=r(c_1)
				local m120_97_5=r(c_2)
			format milk120 %6.0f
		twoway (scatter cf milk120, m(o) msize(tiny) xline(`m120_2_5', lpat(dash)) /// 
			xline(`m120_97_5', lpat(dash))) (lowess cf milk120), scheme(s1mono)  ///
			ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) legend(off)
		graph save figs/fig15_scatter.gph, replace
		graph export figs/fig15_scatter.wmf, replace

*************************************************************************
	* Section 15.6.1 smoothed scatterplots
*************************************************************************
		* determining 2.5 and 97.5 percentiles of -milk120-
		centile  milk120, c(2.5 97.5)
			local m120_2_5=r(c_1)
			local m120_97_5=r(c_2)
		sort milk120
		format milk120 %6.0f
		capture drop lowess8
		lowess cf milk120 , nograph gen(lowess8)
			twoway line lowess8 milk120, ylab(#4) name(lowess8, replace) ///
				ti("Lowess smoother") subtitle("bandwidth=0.8") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
		capture drop line8
		lowess cf milk120 , nograph gen(line8) noweight
			twoway line line8 milk120, ylab(#4) name(line8, replace) ///
				ti("Running line smoother") subtitle("bandwidth=0.8") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
		capture drop mean8
		lowess cf milk120 , nograph gen(mean8) noweight mean
			twoway line mean8 milk120, ylab(#4) name(mean8, replace) ///
				ti("Running mean smoother") subtitle("bandwidth=0.8") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
	* influence of bandwidth
		capture drop lowess2
		lowess cf milk120 , bw(0.2) nograph gen(lowess2)
			twoway line lowess2 milk120, ylab(#4) name(lowess2, replace) ///
				ti("Lowess smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
		capture drop line2
		lowess cf milk120 , bw(0.2) nograph gen(line2) noweight
			twoway line line2 milk120, ylab(#4) name(line2, replace) ///
				ti("Running line smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
		capture drop mean2
		lowess cf milk120 , bw(0.2) nograph gen(mean2) noweight mean
			twoway line mean2 milk120, ylab(#4) name(mean2, replace) ///
				ti("Running mean smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ylab(75(5)95) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
		* combine 6 graphs
			graph combine mean8 line8 lowess8 mean2 line2 lowess2, ///
				col(3) name(smooth_comb, replace) scheme(s1mono)
			graph save "figs/fig15_smoothed.gph", replace
			graph export "figs/fig15_smoothed.wmf", replace
	* adding a confidence interval
		* determining 2.5 and 97.5 percentiles of -milk120-
		centile  milk120, c(2.5 97.5)
			local m120_2_5=r(c_1)			
			local m120_97_5=r(c_2)		
		sort milk120
		capture drop ci8 cise8 ci8ub ci8lb
		running cf milk120, ci gen(ci8) gense(cise8) nograph
			gen ci8ub=ci8+(1.96*cise8)
			gen ci8lb=ci8-(1.96*cise8)
			twoway line ci8 ci8ub ci8lb milk120, ylab(#4) ///
				ti("Running line smoother") subtitle("with 95% CI") note("") ///
				ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none)) ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) ///
				legend(off)
			graph save "figs/fig15_ci.gph", replace
			graph export "figs/fig15_ci.wmf", replace

*************************************************************************
	* smoothed scatterplot on a logistic scale
*************************************************************************
		* determining 2.5 and 97.5 percentiles of -milk120-
		centile  milk120, c(2.5 97.5)
			local m120_2_5=r(c_1)
			local m120_97_5=r(c_2)
		sort milk120
		capture drop lowess_logit
		lowess fs milk120 , nograph gen(lowess_logit) logit
		twoway line lowess_logit milk120, ylab(#4) name(lowess_logit, replace) ///
					ti("Lowess smoother") subtitle("bandwidth=0.8") note("") ///
					ytitle("logit of first service conception", m(r+2)) ylabel(#4, angle(horizontal)) ///
					xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none))  ///
				scheme(s1mono) xline(`m120_2_5', lp(dash)) xline(`m120_97_5', lp(dash)) 
			graph save "figs/fig15_logit.gph", replace
			graph export "figs/fig15_logit.wmf", replace

*************************************************************************
	* Section 15.6.2 categorizing the predictor - indicator variables (results not in text)
*************************************************************************
		sum milk120, detail
		egen m120_c4=cut(milk120), at(0 2500 3000 3500 9999)
		tabstat cf, by(m120_c4) stat( n mean min max)
		reg fs milk120
			estimates store lin
		xi:reg fs i.m120_c4
			estimates store cat
		estimates stats lin cat


*************************************************************************
	* Section 15.6.3 - 16.6.4 Polynomials
*************************************************************************
		use daisy2.dta, clear
	* polynomial models - linear
		regress cf milk120
			predict pr_lin if e(sample)
			label var pr_lin "Linear"
	* polynomials - quadratic
		gen m120_ct=milk120-3000		/* centering the variable before squaring it */
		gen m120_sq=(m120_ct^2)/10000  	/* helps reduce collinearity */
			/* dividing by 10000 scales the parameter estimates to keep them reasonable */
		regress cf m120_ct m120_sq
			predict pr_quad if e(sample)
			label var pr_quad "Quadratic"
	* polynomials - cubic orthogonal
		orthpoly milk120, gen(m120_orth*) degree(3)
		regress cf m120_orth1 m120_orth2 m120_orth3
			predict pr_cubic if e(sample)
			label var pr_cubic "Cubic"
	* polynomials - fractional
		fracpoly reg cf milk120, compare
			predict pr_frac
			label var pr_frac "Fractional"
	* plot linear, quadratic, cubic and fractional polynomial relationships
		* determining 2.5 and 97.5 percentiles of -milk120-
		centile  milk120, c(2.5 97.5)
			local m120_2_5=r(c_1)
			local m120_97_5=r(c_2)
		sort milk120
		twoway line pr_lin pr_quad pr_cubic pr_frac milk120, ylab(#4) name(polynomial, replace) ///
			ti("Polynomial functions") ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none))  scheme(s1mono) legend(off) ///
			text(78 5000 "linear", place(NE)) text(82 6400 "cubic", place(NE)) ///
			text(91 6000 "fractional", place(NE)) text(95 4800 "quadratic", place(NE)) ///
			text(108 1700 "2.5th percentile", place(NE)) text(108 4500 "97.5th percentile", place(NE)) ///
			xline(`m120_2_5', lpattern(dash)) xline(`m120_97_5', lpattern(dash)) 
			graph save "figs/fig15_poly.gph", replace
			graph export "figs/fig15_poly.wmf", replace

*************************************************************************
	* Section 15.6.5 Splines
*************************************************************************
		mkspline m120_linsp 4=milk120, pctile
			reg cf m120_linsp*
			predict linsp, xb
		mkspline m120_cubsp=milk120, cubic knots(2459 2923 3400)
			reg cf m120_cubsp*
			predict cubsp, xb
		twoway line linsp cubsp milk120, ylab(#4)  ///
			ti("Spline functions") ytitle("calving to 1st service (days)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("120 day milk production (kg)", m(t+2)) plotregion(style(none))  scheme(s1mono) legend(off) ///
			text(85 500 "cubic", place(NE)) text(90 400 "linear", place(NE)) ///
			xline(1676, lpattern(dash)) xline(4489, lpattern(dash)) 
			graph save "figs/fig15_spline.gph", replace
			graph export "figs/fig15_spline.wmf", replace

*************************************************************************
* Ex 15.5 Model Selection Procedures
*************************************************************************
	use pig_farm.dta, clear
	gen lnpneu=log(pneu)
	* forward selection model
		stepwise, pe(0.05): regress lnpneu num-exprnce
		estimates store fs
	* backward elimination model
		stepwise, pr(0.05): regress lnpneu num-exprnce
		estimates store be
	* backward stepwise
		stepwise, pr(0.05) pe(0.0499): regress lnpneu num-exprnce
		estimates store sw
	* compare results
		estimates table fs be sw
		estimates stats fs be sw

*************************************************************************
* Ex 15.6 Regression Model Validation
*************************************************************************
	use daisy2.dta, clear
		keep if h7==1
	* generate a random number
		set seed 87654
		gen rand=uniform()
	* fit the final regression model from Chapter 14
		gen wpc_sqrt=wpc^.5	
		gen hs_ct= (herd_size-251)/100
		gen hs_sq = hs_ct^2
		gen parity1 = parity-1
		gen calv_mth=month(calv_dt)
		gen calv_spr=calv_mth>1 & calv_mth<8
		xi: reg wpc_sqrt hs_ct hs_sq parity1 calv_spr twin i.dyst*vag_disch rp if rand<0.5
			predict pv
	* compute the squared correlation between predicted values and observed
	* for the estimation subset and the validation subset
		display "R2 for data used to build the model:   " e(r2)
		corr pv wpc_sqrt if rand>=0.5
		display "R2 for 2nd half of data:   " r(rho)^2

*************************************************************************
* Ex 15.7 Regression Model Presentation - IQR
*************************************************************************
	* refit the BE model for pig data
		use pig_farm.dta, clear
		gen lnpneu=log(pneu)
		replace size=size/1000
		format lnpneu hldbck size exhaust exprnce floor %7.3f
		regress lnpneu hldbck size exhaust exprnce floor
	* compute the IQR for cf
		centile size, centile(25 75)
		di "effect of IQR change in size   " _b[size]*(r(c_2)-r(c_1))
		centile exhaust, centile(25 75)
		di "effect of IQR change in exhaust   " _b[exhaust]*(r(c_2)-r(c_1))
		centile exprnce, centile(25 75)
		di "effect of IQR change in exprnce   " _b[exprnce]*(r(c_2)-r(c_1))
		di "effect of hldbck   " _b[hldbck]
		di "effect of floor   " _b[floor]

