* ch16_all.do
* Logistic regression - all calculations
*
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch16\data		/* this line is computer dependent */
	capture log close ch16_all
	log using ch16_all, name(ch16_all) replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the nocardia dataset
*/


*************************************************************************
* Fig 16.1 Graph of logit and inverse logit functions
*************************************************************************
	twoway (function ln(x/(1-x)), range(0.001 0.999) lcolor(black)) , scheme(s1mono) ///
		ytitle("logit", m(r+2)) ylabel(-8(2)8, angle(horizontal)) ///
		xtitle("probability", m(t+2)) xlabel(0(0.2)1, format(%4.2f)) plotregion(style(none)) ///
		yline(-4.595, lpat(dash) lwidth(medium)) yline(4.595, lpat(dash) lwidth(medium)) name(fig16_logit1, replace)
	twoway (function exp(x)/(1+exp(x)), range(-7 7) lcolor(black)) , scheme(s1mono) ///
		ytitle("probability", m(r+2)) ylabel(0(.2)1, angle(horizontal) format(%4.2f)) ///
		xtitle("logit", m(t+2)) xlabel(-8(2)8) plotregion(style(none)) ///
		xline(-4.595, lpat(dash) lwidth(medium)) xline(4.595, lpat(dash) lwidth(medium)) name(fig16_logit2, replace)
	graph combine fig16_logit1 fig16_logit2, ysize(4) xsize(8)scheme(s1mono)
		graph save "figs/fig16_logit.gph", replace
		graph export "figs/fig16_logit.wmf", replace

*************************************************************************
* Fig 16.2 Graph of log likelihood function
*************************************************************************
	twoway (function ln(comb(10,3)) + 3*ln(x) + 7*ln(1-x), range(0.1 0.6)), ///
		scheme(s1mono) ytitle("log likelihood", m(r+2)) ylabel(-3(1)-1, angle(horizontal)) ///
		xtitle("prevalence", m(t+2)) xlabel(.1(.1).6, format(%4.2f)) plotregion(style(none)) ///
		xline(.3, lpat(dash) lwidth(medium)) 
	graph save "figs/fig16_lnL.gph", replace
	graph export "figs/fig16_lnL.wmf", replace


*************************************************************************
* Ex 16.1 - 16.2 Multiple logistic regression and LRT
*************************************************************************
* open the Nocardia dataset
	use nocardia.dta, clear

* fit a a variety of models
	logit casecont 				/* null */
		estimates store null 
	logit casecont dcpct			/* reduced */
		estimates store reduced			
	logit casecont dcpct dneo dclox	/* full */
		estimates store full
	estimates stats null full reduced
* likelihood ratio tests
	lrtest full null
	lrtest full reduced
* comparing Walds and LRT for -dclox- and -dneo-
	logit casecont dcpct dneo dclox
		test dneo dclox 
* multiple logistic regression - categorical predictor
	xi: logit casecont dcpct dneo dclox i.dbarn
	xi: logistic casecont dcpct dneo dclox i.dbarn		/* results as ORs */


*************************************************************************
* Ex 16.3 Effects on the probability scale
*************************************************************************
* this requires the Stata "add-in" program -spost-
	logit casecont dcpct dneo dclox
		prgen dcpct, gen(pv_dn) from(0) to(100) x(dneo=1) rest(min) ncases(101)
		prgen dcpct, gen(pv_no) from(0) to(100) x(dneo=0) rest(min) ncases(101)
	* generate graph of predicted probabilities
	twoway (line pv_dnp1 pv_dnx, lpat(solid)) ///
		(line pv_nop1 pv_dnx, lpat(dash)), scheme(s1mono) ///
		ytitle("probability of Nocardia", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("percent of cows dry treated", m(t+2)) ///
		legend(off) text(.5 20 "dneo=1") text(.25 70 "dneo=0")
	graph save "figs/fig16_pscale.gph", replace
	graph export "figs/fig16_pscale.wmf", replace


*************************************************************************
* Ex 16.4 - 16.5 Assessment of confounding and interaction
*************************************************************************
* confounding
	logit casecont dcpct dneo dclox
		estimates store full
	logit casecont dneo dclox
		estimates store reduced
	estimates table full reduced, se
* interaction
	xi:logit casecont dcpct i.dneo*dclox

*************************************************************************
* Assessment of linearity of effect of -dcpct- (results not included in text)
*************************************************************************
* unconditional relationship
	lowess casecont dcpct, logit scheme(s1mono) xtitle("percent of cows dry treated", m(t+2)) ///
		ytitle("logit", m(r+2)) ylabel(,angle(horizontal)) ti("") subtitle("") note("") ///
		name(fig16_lin1, replace)
	logit casecont dcpct dneo dclox
		predict p_res, res
		twoway (scatter p_res dcpct) (lowess p_res dcpct, lcolor(black)), ///
			scheme(s1mono) xtitle("percent of cows dry treated", m(t+2)) ///
			ytitle("Pearson residual", m(r+2)) ylabel(-2(2)6, angle(horizontal)) legend(off) name(fig16_lin2, replace)
	* combine the 2 graphs
	graph combine fig16_lin1 fig16_lin2, ysize(4) xsize(8)scheme(s1mono)
		graph save "figs/fig16_lin.gph", replace
		graph export "figs/fig16_lin.wmf", replace
	* fractional polynomials
		fracpoly logit casecont dcpct dneo dclox, compare
		fracpoly logit casecont dcpct , compare
		* polynomial not required for either undonditional or multivariable model

*************************************************************************
* Section 16.11 Generalized Linear Models
*************************************************************************
* fit model with ML logistic regression
	xi:logit casecont dcpct i.dneo*dclox
	estimates store ml1
* fit model with IRLS
	xi:glm casecont dcpct i.dneo*dclox, link(logit) family(binomial) irls
	estimates store irls
* fit model with ML in GLM framework
	xi:glm casecont dcpct i.dneo*dclox, link(logit) family(binomial)
	estimates store ml2
* fit model with ML in GLM framework and robust SE
	xi:glm casecont dcpct i.dneo*dclox, link(logit) family(binomial) vce(robust)
	estimates store robust
estimates table ml1 irls ml2 robust, se

	xi:glm casecont dcpct i.dneo*dclox, link(logit) family(binomial) scale(dev) irls

*************************************************************************
* Evaluating a Logistic Regression Model
*************************************************************************
	use nocardia.dta, clear
* all diagnostics will be based on a model containing -dcpct- , -dneo- , -dclox-  and their interaction
	xi:logit casecont dcpct i.dneo*dclox

* examining the covariate patterns
	predict cov, num
	predict pv, p
	predict res_p, res
	gen res_pi=(casecont-pv)/(pv*(1-pv))^.5
	sort cov
	* generate a count of the number of obs. in each cov. pattern and a counter
		by cov: gen cnt=_N
		by cov: gen seq=_n
		by cov: egen obs=total(casecont)
		by cov: gen obs_p=obs/_N
	format cnt seq obs %3.0f
	format obs_p res_p %6.3f
	list cov cnt dcpct dneo dclox pv casecont obs obs_p in 1/13, clean noobs
	list cov id casecont dcpct dneo dclox pv obs_p res_p res_pi if cov==9, clean noobs

*************************************************************************
* Ex 16.7 - 16.10 Evaluating overall fit
*************************************************************************
	use nocardia.dta, clear
	xi:logit casecont dcpct i.dneo*dclox
* Pearson and Deviance goodness-of-fit tests
	estat gof
	xi:logistic casecont dcpct i.dneo*dclox
		ldev
* Hosmer - Lemeshow Test
	estat gof, g(10) table
* quantifying dispersion
	xi:glm casecont dcpct i.dneo*dclox, link(logit) family(binomial)
* r2 and other fit statistics
	xi:logit casecont dcpct i.dneo*dclox
	fitstat	/* fitstat is a Stata add-on */
* sensitivity and specificty of logistic model
	estat classification
	* changing the cutpoint
		lstat, cut(0.25)
* two graph ROC
	lsens, lpattern(solid dash) scheme(s1mono) lcolor(black black) mcolor(black black) ///
		legend(off) ylabel(#5, angle(horizontal) format(%4.2f))  ///
		text(.85 .1 "sensitivity", place(NE)) text(.8 .8  "specificity", place(NE)) ///
		ytitle(, m(r+2)) xtitle(, m(t+2)) plotregion(style(none)) 
	graph save "figs/fig16_2groc.gph", replace
	graph export "figs/fig16_2groc.wmf", replace

* standard ROC graph
	lroc , scheme(s1mono) lcolor(black black) mcolor(black black) ///
		ylabel(#5, angle(horizontal) format(%4.2f)) ///
		ytitle(, m(r+2)) xtitle(, m(t+2)) plotregion(style(none)) ///
		note("")
	graph save "figs/fig16_roc.gph", replace
	graph export "figs/fig16_roc.wmf", replace


*************************************************************************
* Section 16.12.4 and Ex. 16.8 Overdispersion
*************************************************************************
* simple example
	clear
	set seed 555777
	set obs 10
	gen grp=_n
	gen lv=(grp>=6)
	expand 20
	genbinomial y1, mu(.4)
	gen y2=(grp<=4)
	sum y1 y2
	collapse (mean) y1 y2, by(grp)
	sum y1 y2
	list grp y1 y2, clean noobs

* detecting overdispersion example
	clear
	set seed 555777
	set obs 10
	gen grp=_n
	gen hp=(grp>=6)	/* identify 5 high prevalence groups */
	gen x=(grp==1 | grp==2 | grp==6 | grp==7)	
				/* group level variable - 2 high and 2 low prev. groups */
	gen xb= -2 + (3*hp) + x
	expand 100
 	genbinomial y, xbeta(xb)
		/* note: results vary from run to run because of variability introduced by genbinomial */
	* fit model with group level variable that is found equally in high and low prev. groups
		glm y x, fam(binomial) link(logit)
		logit y x
		lfit
		* no evidence of overdispersion or lack of fit
	* collapse the data to grouped (binomial) format and re-analyse
		preserve
		collapse (mean) x (count) n=y (sum) cases=y, by(grp)
		glm cases x, fam(binomial n) link(logit)
		restore


*************************************************************************
* Ex 16.11 - 16.12 Evaluating residuals - outliers and influential points
*************************************************************************
	use nocardia.dta, clear
* all diagnostics will be based on a model containing -dcpct- , -dneo- , -dclox-  and their interaction
	xi:logit casecont dcpct i.dneo*dclox
* examining Pearson and Deviance residuals
	predict pv
	predict cov, num
	sort cov
	* generate a count of the number of obs. in each cov. pattern and a counter
		by cov: gen cnt=_N
		by cov: gen seq=_n
		by cov: egen obs=total(casecont)
		by cov: gen obs_p=obs/_N
		format cnt seq obs %3.0f
		format obs_p %6.3f
	predict res_p, res
	predict res_d, dev
	predict res_stdp, rsta
	predict lev, hat
	predict db, dbeta
	predict dx2, dx2
	predict ddev, ddev
	format pv res_p res_d res_stdp lev db dx2 ddev %6.3f
	* scatterplot by covariate pattern
		twoway (scatter res_stdp cov [aweight=cnt], msymbol(Oh) mlcolor(black) mlwidth(medium)) ///
		(scatter res_stdp cov, msize(vtiny) mlabel(cov)), scheme(s1mono) ///
		ytitle("standardized Pearson residual", m(r+2)) ylabel(-2(2)6, angle(horizontal) format(%4.1f)) ///
		xtitle("covariate pattern", m(t+2)) plotregion(style(none)) legend(off)
	graph save "figs/fig16_p_res.gph", replace
	graph export "figs/fig16_p_res.wmf", replace
* list extreme residuals
	sort res_stdp
	list cov cnt dcpct dneo dclox pv res_p res_stdp if res_stdp<= -2 & seq==1, clean noobs abb(6)
	list cov cnt dcpct dneo dclox pv res_p res_stdp if res_stdp>= 2 & seq==1, clean noobs abb(6)
	* refitting the model without cov. pattern #7
		xi:logit casecont dcpct i.dneo*dclox
			estimates store all
		xi:logit casecont dcpct i.dneo*dclox if cov!=7
			estimates store not_cp7
		estimates table all not_cp7, se

* identifying highest leverage points
	sort lev
	list cov cnt dcpct dneo dclox pv obs_p res_p lev if lev>0.6 & seq==1, clean noobs abb(6)

* Cooks distance
	xi:glm casecont dcpct i.dneo*dclox, fam(bin) link(logit)
	predict cooksd, cooksd
	sort cooksd
	list cov cnt dcpct dneo dclox pv obs_p res_p cooksd if cooksd>0.02 & seq==1, clean noobs abb(6)

* evaluating delta betas
	sort db
	list cov cnt dcpct dneo dclox pv obs_p res_p db if db>1 & seq==1, clean noobs abb(6)
	* cov. pattern 28 has a huge influence on the coefficient(s)
	* refit the model without those observations
		xi:logit casecont dcpct i.dneo*dclox if cov!=28	/* nonsensical results */
		* effect of covariate pattern on model without interaction
			xi:logit casecont dcpct i.dneo*dclox if cov!=28
				estimates store not_cp28
			estimates table all not_cp28, se
			* no particularly large impact

* evaluating delta chisq and delta dev
	sort dx2
	list cov cnt dcpct dneo dclox pv obs_p res_p dx2 ddev if (dx2>2 | ddev>2) & seq==1, clean noobs abb(6)

*************************************************************************
* Section 16.13 Sample size
*************************************************************************

* assume that you want to doa cohort sutdy in which the main risk factor 
* of interest is expected to raise the risk of disease from 2% to 5%
	sampsi .05 .02, power(.8)
		* you need 653 exposed and 653 non-exposed subjects
* adjustment for clustering
	* assume that the risk factor is a herd level factor 
	* and avg. herd size = 100 cows), and that the estimated ICC is 0.2
		sampclus , obs(100) rho(.2)
			* you need 272 herds (data on 27,200 cows)
			* if you just sample 10 cows from each herd
			sampclus , obs(10) rho(.2)
				* now you need 366 herds (183 exposed and 183 non-exposed)
				* ... but only a total of 3660 cows
* adjustment for multiple variables
	* assume that you think there will be 3 other factors that are likely 
	* to be confounders and that the r2 of these confounders when 
	* predicting the main risk factor is .25
		di 366 * ( 1 / (1-0.25) )
			* you now need 488 herds (244 E+ and 244 E-)

*************************************************************************
* Ex 16.13 Exact Logistic Regression
*************************************************************************
clear
set memory 500m
use nocardia.dta
	xi:exlogistic casecont  i.dneo*dclox , condvars(dcpct) estc coef memory(500m)
	xi:exlogistic casecont  i.dneo dclox , condvars(dcpct) estc coef memory(500m)
	xi:logit casecont dcpct i.dneo*dclox , robust
	xi:logit casecont dcpct i.dneo*dclox


*************************************************************************
* Ex 16.15 - 16.16 Conditiional Logistic Regression
*************************************************************************
	use sal_outbrk.dta, clear

* lack of information from groups with no within-group variation
	list match_grp casecontrol eatpork eatbeef slt_a dlr_a if match_grp==23, clean noobs

* Mantel-Haenszel statistics for binary exposures (for comparison purposes)
	cc casecontrol slt_a, by(match_grp) nohom  /* OR=3.87, P=0.002 */

* conditional logistic regression
	logistic casecontrol slt_a 		/* incorrect analysis */
	clogit casecontrol slt_a, group(match_grp) or 
		estimates store simple
	xi: clogit casecontrol i.slt_a*gender, or group(match_grp)
		*note gender dropped because no within group variance
		estimates store multi
	lrtest simple multi

* conditional logistic models - diagnostics
	clogit casecontrol slt_a, group(match_grp) or
		capture drop  _hat _dx2 _dbeta _gdx2 _gdbeta
	clfit  /* Stata add-on for CLR diagnostics */
		sort _gdbeta match_grp casecontrol
		format _hat _dx2 _dbeta _gdx2 _gdbeta %5.3f
		list match_grp casecont slt_a _hat _dx2 _dbeta _gdx2 _gdbeta in -11/l, clean noobs
		twoway scatter _gdx2 _gdbeta, 
		* match_grp 2 and 9 have big influence on dbeta, refit model without
			clogit casecontrol slt_a if match_grp~=2 & match_grp~=9, group(match_grp) or


