* ch17_all.do
* Multinomial and ordinal regression - all calculations
*
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch17\data		/* this line is computer dependent */
	capture log close ch17_all
	log using ch17_all, name(ch17_all) replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the nocardia dataset
*/

*************************************************************************
* Table 17.2 Cross-tabulation of -bckgrnd- and -grade-
*************************************************************************
* open the beef ultrasound dataset
	use beef_ultra.dta, clear
	tab grade bckgrnd
	cci 15 149 79 198
	cci 15 149 26 20
	egen pom1=cut(grade), at(1 2 99) icodes
		cc pom1 bckgrnd
	egen pom2=cut(grade), at(1 3 99) icodes
		cc pom2 bckgrnd
	* unconditional adjacent category model
		constraint define 1 [AA]bckgrnd=0.5*[A]bckgrnd
		mlogit grade bckgrnd, baseoutcome(1) constraint(1) rrr

*************************************************************************
* Ex 17.1 Simple multinomial logistic regression
*************************************************************************
* Fit a simple multinomial logistic model
	mlogit grade bckgrnd, baseoutcome(1)
	mlogit grade bckgrnd, baseoutcome(1) rrr
		mlogtest, hausman
		mlogtest, smhsiao
		mlogtest, lrcomb
	* model based on -sex-
		mlogit grade sex, baseoutcome(1) rrr
		test sex

*************************************************************************
*Ex 17.2  Multiple multinomial logistic regression
*************************************************************************
* Fit a multiple multinomial logistic model
	mlogit grade bckgrnd sex backfat ribeye imfat carc_wt , baseoutcome(1)
		estimates store mlogit
		mlogtest, hausman
		mlogtest, smhsiao
		mlogtest, lrcomb
* Compute predicted probabilites 
	mlogit grade bckgrnd sex backfat ribeye imfat carc_wt, b(1) 
		predict pA pAA pAAA, p
		format pA pAA pAAA %5.3f
		sort id
		list id grade bckgrnd sex backfat ribeye imfat carc_wt pAAA pAA pA in 1/5, ///
			clean noobs string(5) ab(5)
* Statistical significance of -sex-
	* Wald test
		test sex
	* LRT
		mlogit grade bckgrnd backfat ribeye imfat carc_wt , baseoutcome(1)
		lrtest . mlogit


*************************************************************************
* Ex 17.4 Proportional odds model
*************************************************************************
	use beef_ultra.dta, clear
* fit a proportional odds model
	ologit grade bckgrnd sex backfat ribeye imfat carc_wt , table
		estimates store ologit
	* compute predicted values for all individuals
		capture drop pA pAA pAAA 
		predict pAAA pAA pA, p
		predict xb, xb
		format xb pA pAA pAAA %5.3f
	* list the relevant variables for the first 5 observations
		sort id
		format grade %5.0g
		list grade bckgrnd sex backfat ribeye imfat carc_wt xb pAAA pAA pA in 1/5, ///
			clean noobs string(5) ab(5)
* generate smoothed values of predicted probabilities
	capture drop pAsm pAAsm pAAAsm
	lowess pAAA imfat if imfat>3 & imfat<6, bw(.5) gen(pAAAsm)
	lowess pAA imfat if imfat>3 & imfat<6, bw(.5) gen(pAAsm)
	lowess pA imfat if imfat>3 & imfat<6, bw(.5) gen(pAsm)
		lab var pAAAsm "Grade AAA"
		lab var pAAsm "Grade AA"
		lab var pAsm "Grade A"
	* graph the smoothed predicted values against imfat
		sort imfat
		twoway line pAAAsm pAAsm pAsm imfat if imfat>3 & imfat<6, scheme(s1mono) ///
			lcolor(black black black) lpattern(solid dash dash_dot) legend(off) ///
			ytitle("probability of grade", m(r+2)) ylabel(0 .2 .4 .6, angle(horizontal) format(%4.2f)) ///
			xtitle("intramuscular fat score (%)", m(t+2)) xlabel(3 4 5 6,format(%4.2f)) ///
			plotregion(style(none)) text(.6 4.5 "AA", place(NE)) ///
			text(.3 4.5 "AAA", place(NE)) text(.1 4.5 "A", place(NE)) 
 		graph save "figs/fig17_ord_prb.gph", replace
		graph export "figs/fig17_ord_prb.wmf", replace

*************************************************************************
* Section 17.5.2 Evaluating the proportional odds assumption
*************************************************************************
* approximate likelihood ratio test using stored estimates
	* comparison with multinomial model
		lrtest ologit mlogit, stats force
	* comparison with generalized ordinal model
		gologit2 grade bckgrnd sex backfat ribeye imfat carc_wt
		estimates store gologit
		lrtest ologit gologit, stats force
* approximate likelihood ratio test using -omodel-
	omodel logit grade bckgrnd sex backfat ribeye imfat carc_wt 
* Brant test
	ologit grade bckgrnd sex backfat ribeye imfat carc_wt , table
	brant, detail

*************************************************************************
* Section 17.5.3 Dealing with non-proportional odds 
*************************************************************************
* generalized ordinal logistic regression	
	gologit2 grade bckgrnd sex backfat ribeye imfat carc_wt 
* partial proportional odds
	gologit2 grade bckgrnd sex backfat ribeye imfat carc_wt , autofit
	* allowing slopes to vary for -bckgrnd- and -ribeye-
	gologit2 grade bckgrnd sex backfat ribeye imfat carc_wt , pl(carc_wt backfat sex imfat)
		estimates store partial
		lrtest ologit partial, force stats
* stereotype logistic model (not included in text)
	slogit grade bckgrnd sex backfat ribeye imfat carc_wt
		estimates store slogit
* heterogenous choice logistic model (not included in text)
	oglm grade bckgrnd sex backfat ribeye imfat carc_wt, het(bckgrnd ribeye) store(oglm)
		lrtest ologit oglm, stats force

*************************************************************************
* Ex 17.5 Adjacent category model (constrained multinomial model)
*************************************************************************
* define the constraintes
	local list bckgrnd sex backfat ribeye imfat carc_wt 
	local i=1
	foreach var of local list {
	constraint define `i' [AA]`var'=0.5*[A]`var'
	local i=`i'+1
		}
* fit constrained model
	mlogit grade bckgrnd sex backfat ribeye imfat carc_wt , b(1) constraint(1-6)
		estimates store acm
		lrtest mlogit acm, stats

*************************************************************************
* Section 17.7 Continuation ratio model (for demonstration of coding only)
*************************************************************************
* first create the two new outcome variables
	gen gr2=(grade==2)
		replace gr2=. if grade>2
	gen gr3=(grade==3)
* fit an unconstrained model
   * fit the logistic models and store the log likelihoods
	logit gr2 bckgrnd sex backfat ribeye imfat carc_wt
		scalar ll2=e(ll)
	logit gr3 bckgrnd sex backfat ribeye imfat carc_wt
		scalar ll3=e(ll)
	display ll2 + ll3
* fit a constrained model
	ocratio grade bckgrnd sex backfat ribeye imfat carc_wt

