
* ch18_all.do
* Modelling count and rate data - all calculations
*and a few analyses that are not included in the chapter
	clear
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch18\data		/* this line is computer dependent */
	capture log close ch18_all
	log using ch18_all, name(ch18_all) replace 

*************************************************************************
* Fig 18.1 Poisson distributions
	clear
	cd c:\ver2\ch18\data
	set obs 100000
	set seed 98765
	genpoisson p_05, mu(0.5)
	genpoisson p_10, mu(1.0)
	genpoisson p_20, mu(2.0)
	genpoisson p_50, mu(5.0)
	sum

histogram p_05, width(1) start(0) fraction discrete ///
	barwidth(0.1) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)4) ///
	mcolor(black) ytitle("probability of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 0.5", size(medium)) ylabel(#4, angle(horizontal)) 
	graph save "C:\ver2\ch18\data\fig18_pd05.gph", replace

histogram p_10, width(1) start(0) fraction discrete ///
	barwidth(0.125) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)5) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 1.0", size(medium)) ylabel(0 .1 .2 .3 .4, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_pd10.gph", replace

histogram p_20, width(1) start(0) fraction discrete ///
	barwidth(0.2) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)9) ///
	mcolor(black) ytitle("probability of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 2.0", size(medium)) ylabel(#4, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_pd20.gph", replace

histogram p_50, width(1) start(0) fraction discrete ///
	barwidth(0.3) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(2)16) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 5.0", size(medium)) ylabel(0 .05 .1 .15 .2, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_pd50.gph", replace

* combine the four graphs
graph combine fig18_pd05.gph fig18_pd10.gph fig18_pd20.gph fig18_pd50.gph , ///
	scheme(s1mono) plotregion(style(none)) 
	graph save "C:\ver2\ch18\figs\fig18_poisson_dist.gph", replace
	graph export "C:\ver2\ch18\figs\fig18_poisson_dist.wmf", replace

*************************************************************************
* Ex 18.1 Fitting a Poisson model - ML estimation
*************************************************************************
	use c:\ver2\data\tb_real.dta, clear
	xi: poisson reactors i.type sex i.age, exp(par)
		test _Itype_2 _Itype_3 _Itype_4
	* calculations of incidence rates
     		display exp(_b[_cons])
      	display exp(_b[_cons] + _b[_Iage_1])
      	display exp(_b[_cons] + _b[_Itype_3])
      	display exp(_b[_cons] + _b[_Iage_1] + _b[_Itype_3])
	* fitting a Poisson - GLM framework
		xi: glm reactors i.type sex i.age, exp(par) fam(poi)
	* expressing results as IRR
		xi: glm reactors i.type sex i.age, exp(par) fam(poi) eform
	* Goodness of fit test
		xi: poisson reactors i.type sex i.age, exp(par)
			estat gof
			estat gof, pearson

*************************************************************************
* Fig 18.2 Computing predicted values and comparing to observed
*************************************************************************
	xi: poisson reactors i.type sex i.age , exp(par)
	* using SPOST (-prcounts-) to compute predictred probabilities
	* Note: SPOST is a user written add-on to Stata
	* -prcounts- doesn't work after -glm- so I used the -poisson- command
		capture drop pc*
		prcounts pc, plot max(30) 
		twoway line pcpreq pcobeq pcval if pcval<=10, ///
			scheme(s1mono)lcolor(black black) ///
			lpattern(solid dash) title("") legend(off) ///
			ytitle("proportion of observations", m(r+2)) ///
			xtitle("count", m(t+2)) plotregion(style(none)) ///
			text(.09 0 "predicted", place(NE)) text(.13 1.4  "observed", place(NE))
		graph save "C:\ver2\ch18\figs\fig18_pred_prob.gph", replace
		graph export "C:\ver2\ch18\figs\fig18_pred_prob.wmf", replace
		capture drop pc*

*************************************************************************
* Section 18.5.3 Dispersion parameters
*************************************************************************
	xi: glm reactors i.type sex i.age, exp(par) fam(poi)
	* Score and Lagrange multiplier tests (not in text)
		gen cens=1
		xi: cpoisson reactors i.type sex i.age, censor(cens) exp(par)
 
*************************************************************************
* Ex 18.2 Poisson regression diagnostics
*************************************************************************
	* refit the model using -glm- in order to obtain scaled SE, and
	* to get a wider range of diagnostic prameters
	* scaled SE
		xi: glm reactors i.type sex i.age, exp(par) fam(poisson) link(log) scale(x2)
	* compute predicted values, residuals and Cook's distance
		xi: glm reactors i.type sex i.age, exp(par) fam(poisson) link(log)
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe standardized
		predict cooks_d, cooksd
		predict mu, mu
		* br obs farm_id reactors par type sex age mu res_d res_p res_a cooks_d
	* list the smallest and largest residuals and the largest Cook's Distances
		sort res_a
		list  obs type sex age par reactors mu in 1/3, clean noobs
		list  obs res_d res_p res_a cooks_d in 1/3, clean noobs
		list  obs type sex age par reactors mu in -3/-1, clean noobs
		list  obs res_d res_p res_a cooks_d in -3/-1, clean noobs
		sort cooks_d
		list  obs type sex age par reactors mu in -4/-1, clean noobs
		list  obs res_d res_p res_a cooks_d in -4/-1, clean noobs
	* diagnostic plots
		twoway scatter res_a mu, mlabel(obs) saving(r1.gph, replace) ///
			scheme(s1mono) xtitle("Predicted count") ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 	
		twoway scatter res_a cooks_d, mlabel(obs) saving(r2.gph, replace) ///
			scheme(s1mono) xtitle("Cook's distance") ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 	
		* combine the 2 residuals graphs
			graph combine r1.gph r2.gph , xsize(6) ysize(3) ///
			subtitle("Diagnostic plots - Poisson regression")
		graph save "C:\ver2\ch18\figs\fig18_pois_diag_plots.gph", replace
		graph export "C:\ver2\ch18\figs\fig18_pois_diag_plots.wmf", replace

*************************************************************************
* Fig 18.4 Negative binomial distributions
*************************************************************************
*generate variables with negative binomial distributions
	clear
	cd c:\ver2\ch18\data
	set obs 100000
	set seed 98765
	gennbreg nb_2_1, mu(2.0) alpha(1)
	gennbreg nb_2_2, mu(2.0) alpha(2)
	gennbreg nb_5_1, mu(5.0) alpha(1)
	gennbreg nb_5_2, mu(5.0) alpha(2)
	sum

keep if nb_2_1<=12
histogram nb_2_1, width(1) start(0) fraction discrete ///
	barwidth(0.1) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)12) ///
	mcolor(black) ytitle("proportion of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 2.0, alpha = 1", size(medium)) ylabel(0(.1).5, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_nb_2_1.gph", replace

keep if nb_2_2<=24
histogram nb_2_2, width(1) start(0) fraction discrete ///
	barwidth(0.2) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(2)24) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 2.0, alpha = 2", size(medium))ylabel(0(.1).5, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_nb_2_2.gph", replace

keep if nb_5_1<=36
histogram nb_5_1, width(1) start(0) fraction discrete ///
	barwidth(0.3) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(3)36) ///
	mcolor(black) ytitle("proportion of observations",m(r+2)) plotregion(style(none)) ///
	ti("mean = 5.0, alpha = 1", size(medium)) ylabel(0(.1).3, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_nb_5_1.gph", replace

keep if nb_5_2<=56
histogram nb_5_2, width(1) start(0) fraction discrete ///
	barwidth(0.4) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(4)56) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 5.0, alpha = 2", size(medium)) ylabel(0(.1).3, angle(horizontal))
	graph save "C:\ver2\ch18\data\fig18_nb_5_2.gph", replace

* combine the four graphs
graph combine fig18_nb_2_1.gph fig18_nb_2_2.gph fig18_nb_5_1.gph fig18_nb_5_2.gph , ///
	scheme(s1mono) plotregion(style(none)) 
	graph save "C:\ver2\ch18\figs\fig18_nb_dist.gph", replace
	graph export "C:\ver2\ch18\figs\fig18_nb_dist.wmf", replace

*************************************************************************
* Negative binomial regression - all calculations
*************************************************************************
	set more off
	use c:\ver2\data\tb_real.dta, clear

*************************************************************************
* Evaluating the relationship between mean and var. of reactors (not in text)
*************************************************************************
	sum reactors
	table age sex, contents(mean reactors sd reactors)

*************************************************************************
* Ex 18.3 - 18.4 Comparing Negative binomial models fit using ML and GLM
*************************************************************************
	xi: nbreg reactors i.type sex i.age, exp(par)
		estimates store nb_ml
	xi: glm reactors i.type sex i.age, fam(nb 1.740375) link(log) exp(par)
		/* note: -glm- does not estimate "alpha" so to get equivalent results
		it needs to be estimated using -nbreg- and then added to the -family- option */
	di chi2tail(127, 99.4)    /* deviance based goodness of fit */
	di chi2tail(127, 374.9)   /* Pearson goodness of fit test */
		estimates store nb_glm
		estimates table nb_ml nb_glm, b(%5.3f) se(%5.3f)

*************************************************************************
* Ex 18.5 Model diagnostics
*************************************************************************
	xi: glm reactors i.type sex i.age, fam(nb 1.740375) link(log) exp(par)
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe
		predict cooks_d, cooksd
		predict mu, mu
		sort cooks_d
	br obs farm_id reactors par type sex age mu res_d res_p res_a cooks_d
	list  obs type sex age par reactors mu in -4/-1, clean noobs
	list  obs res_d res_p res_a cooks_d in -4/-1, clean noobs
	* diagnostic plots
		twoway scatter res_a mu, mlabel(obs) saving(r1.gph, replace) ///
			scheme(s1mono) xtitle("Predicted count", size(large)) ///
			ytitle("Anscombe residuals", m(t+2) size(large)) plotregion(style(none)) 	
		twoway scatter res_a cooks_d, mlabel(obs) saving(r2.gph, replace) ///
			scheme(s1mono) xtitle("Cook's distance",  size(large)) ///
			ytitle("Anscombe residuals", m(t+2) size(large)) plotregion(style(none)) 	
		* combine the 2 residuals graphs
			graph combine r1.gph r2.gph , scheme(s1mono) xsize(6) ysize(3) ///
			subtitle("Diagnostic plots - Negative binomial regression")
		graph save "C:\ver2\ch18\figs\fig18_nb_diag_plots.gph", replace
		graph export "C:\ver2\ch18\figs\fig18_nb_diag_plots.wmf", replace
	* fitting the model with and without the largest Cook's D
		xi: nbreg reactors i.type sex i.age, exp(par)
		xi: nbreg reactors i.type sex i.age if obs!= 133, exp(par)

*************************************************************************
* Ex 18.6 Generalized negative binomial regression
*************************************************************************
	xi: nbreg reactors i.type sex i.age, exp(par)
		estimates store nb2
	xi: gnbreg reactors i.type sex i.age, exp(par) lnalpha(i.age)
		estimates store nbh
	estimates stats nb2 nbh
	di chi2tail(2, (2*(-155.154 + 157.736)))

*************************************************************************
* Ex 18.7 Zero inflated model - FEC data
*************************************************************************
	use c:\ver2\data\fec.dta, clear
	xi:zinb fec i.lact past_lact man_heif man_lact if tx==0, ///
		inflate(i.lact past_lact man_heif man_lact) nolog vce(cluster cow)
	estimates store zinfl
	* compute the vuong statistic
	xi:zinb fec i.lact past_lact man_heif man_lact if tx==0, ///
		inflate(i.lact past_lact man_heif man_lact) nolog vuong

*************************************************************************
* Ex 18.8 Hurdle (negative binomial, logit) model - FEC data
*************************************************************************
	xi:hnblogit fec i.lact past_lact man_heif man_lact if tx==0, ///
		nolog vce(cluster cow)
		estimates store hrdl
	* comparison of zero-inflated and hurdle models
		estimates stats zinfl hrdl
		estimates table zinfl hrdl

*************************************************************************
* Ex 18.9 Zero truncated model - SPC data
*************************************************************************
	use c:\ver2\data\daisy2.dta, clear
	keep if h7==1
	keep region herd cow parity cf twin dyst rp vag_disch spc
	xi: ztnb spc parity cf vag_disch , vce(cluster herd)
		estimates store ztnb
	* for comprison, fit an ordinary neg. bin. model to spc-1
	gen spc_1=spc-1
	xi: nbreg spc_1 parity cf vag_disch , vce(cluster herd)
		estimates store nb
		estimates stats ztnb nb

*************************************************************************
* alternative approaches to dealing with overdispersion due to clustering
* results not shown in text
*************************************************************************
	use c:\ver2\data\tb_real.dta, clear
	set more off
	xtset farm_id
	* no adjustment for clustering (for comparison puposes)
		xi: poisson reactors i.type sex i.age, exp(par) 
			estimates store p
		xi: nbreg reactors i.type sex i.age, exp(par) 
			estimates store nb
	* fixed effects of farm
		xi: poisson reactors i.type sex i.age i.farm_id, exp(par) 
			estimates store fe_p
		xi: nbreg reactors i.type sex i.age i.farm_id, exp(par) 
			estimates store fe_nb
	* GEE
		xi: xtgee reactors i.type sex i.age, exp(par) link(log) fam(poisson) corr(exch)
			estimates store gee_p
		* for NB ... first obtain estimate of alpha ignoring clustering
			xi: nbreg reactors i.type sex i.age, exp(par) 
			xi: xtgee reactors i.type sex i.age, exp(par) link(log) fam(nb 1.740375) corr(exch)
				estimates store gee_nb
	* random effects of farm
		xi: xtpoisson reactors i.type sex i.age, exp(par) 
			estimates store xt_p
		xi: xtnbreg reactors i.type sex i.age, exp(par) 
			estimates store xt_nb
		estimates stats p nb fe_p fe_nb gee_p gee_nb xt_p xt_nb
		estimates table p nb fe_p fe_nb gee_p gee_nb xt_p xt_nb




