* ch19_all.do
* Modelling Survival Data - all exercises and figures
*
	clear all
	version 10
	set more off 
	set memory 50m
	cd c:\ver2\ch19\data		/* this line is computer dependent */
	capture log close ch19_all
	log using ch19_all, name(ch19_all) replace 

*************************************************************************
* Figure 19.1 - Histogram - Prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* generate a histogram
	histogram dar, freq width(30) scheme(s1mono) ///
		ytitle("frequency", m(r+2)) ylabel(,angle(horizontal))   ///
		xtitle("days to conception", m(t+2)) 
	graph save "C:\ver2\ch19\figs\fig19_histo_prgl.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_histo_prgl.wmf", replace


*************************************************************************
* Example 19.2 Actuarial and Kaplan-Meier estimates of survivor functions
* and Fig 19.5 and 19.6 graph of survivor cumulative hazard functions
*************************************************************************
* open the calf pneumonia dataset
	use calf_pneu.dta, clear
* generate an actuarial life table (survival and hazard)
	ltable days pn, interval(15) 
	ltable days pn, interval(15) hazard
* generate a Kaplan-Meier estimate of the survival function
	stset days, f(pn)
	sts list
* graph the cumulative survivor function
	sts graph, gwood ciopts(lpattern(dash) lcolor(black)) ///
		legend(off) scheme(s1mono) title("") plotregion(style(none)) ///
		ytitle("cumulative survival probability", m(r+2)) ///
		xtitle("time (days)", m(r+2)) ylabel(,angle(horizontal))  
	graph save "c:\ver2\ch19\figs\fig19_srv_km", replace
	graph export "c:\ver2\ch19\figs\fig19_srv_km.wmf", replace
* graph the Nelson - Aalen cumulative hazard function
	sts graph, na ytitle("cumulative hazard", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("time (days)", m(t+2)) plotregion(style(none))  ///
		cna ciopts(lpattern(dash) lcolor(black)) scheme(s1mono) legend(off)		
	graph save "C:\ver2\ch19\figs\fig19_srv_ch.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_srv_ch.wmf", replace


*************************************************************************
* Example 19.4 Non-parametric comparison of survivor functions
*************************************************************************
* open the calf pneumonia dataset
	use calf_pneu.dta, clear
	stset days, f(pn)
* log-rank test
	sts test stock
* M-H analysis
	* computed by hand using formula in Ch 13


*************************************************************************
* Example 19.4 and Fig 19.7 Non-parametric comparison of survivor functions
*************************************************************************
* open the calf pneumonia dataset
	use calf_pneu.dta, clear
* generate an actuarial life table (survival and hazard)
	stset days, f(pn)
* generate survivor functions for the two stocking groups
	sts graph, by(stock) ylabel(,angle(horizontal)) ///
		legend(off) scheme(s1mono) ytitle("cumulative survival probability") ///
		text(.60 125 "batch", place(ne)) text(.32 120 "continuous", place(ne)) ///
		title("") plotregion(style(none)) ///
		plot2opts(lcolor(black)) ylabel(,glwidth(medthick) glcolor(black) glpattern(dot)) 
	graph save "c:\ver2\ch19\figs\fig19_nonpar_tests", replace
	graph export "c:\ver2\ch19\figs\fig19_nonpar_tests.wmf", replace
* run non-parametric tests of equivalence
	sts test stock
	sts test stock, wilcoxon
	sts test stock, cox
	sts test stock, tware
	sts test stock, peto
	

*************************************************************************
* Figure 19.8 Failure, Survivor and density functions
*************************************************************************
	clear
* create time scale, set hazard to 0.01 and compute S(t)
	set obs 501
	gen t=_n-1
	scalar h=0.01
	gen S=exp(-h*t)
* create intervals for step function, and S(t) - step
	gen temp=mod(t,50)
	gen temp2=_n+25
		gen S_step=S
		replace S_step=. if temp~=0
		replace S_step=S_step[temp2] if _n<=475
		replace S_step=1 in 1
* create continuous failure function F(t)
	gen F=1-S
* create step F(t)
		gen F_step=F
		replace F_step=. if temp~=0
		replace F_step=F_step[temp2] if _n<=475
		replace F_step=0 in 1
* compute derivative of F(t)
	dydx F t, gen(f)
	gen slope=F[150] + f[150]*(t-150)
		replace slope=. if t<50 | t>250
* produce the graph
	twoway (scatter S_step t, connect(stairstep) lpattern(dash) msymbol(none) ) ///
		(line F t, lpattern(dash) lcolor(black)) ///
		(scatter F_step t, connect(stairstep) lpattern(solid) msymbol(none) ) ///
		(line slope t, lcolor(black)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("cumulative probability of survival / failure", m(r+2)) ///
		xtitle("analysis time", m(t+2)) ylabel(,angle(horizontal)) 
	graph save "c:\ver2\ch19\figs\fig19_fct", replace
	graph export "c:\ver2\ch19\figs\fig19_fct.wmf", replace


*************************************************************************
* Figure 19.9 - Exponential Survival Function
*************************************************************************
	clear
* compute S(t)
	set obs 501
	gen t=_n-1
	gen S=exp(-0.01*t)
* produce graph
	twoway (line S t, lcolor(black)), scheme(s1mono) legend(off) ///
		ytitle("cumulative probability of survival", m(r+2)) ///
		xtitle("analysis time", m(t+2)) plotregion(style(none)) ylabel(,angle(horizontal))  
	graph save "c:\ver2\ch19\figs\fig19_srv_exp", replace
	graph export "c:\ver2\ch19\figs\fig19_srv_exp.wmf", replace


*************************************************************************
* Figure 19.10 - Weibull Hazard Function
*************************************************************************
* create a dataset of 101 observations, time scale of 0 to 10
	clear
	set obs 101
	gen t=(_n-1)/10
* set various hazards (p = 0.5, 1, 1.5, 2 and 3) (lambda=1)
	gen h_5=0.5*(t^(0.5-1))
	gen h1=1*(t^(1-1))
	gen h1_5=1.5*(t^(1.5-1))
	gen h2=2*(t^(2-1))
	gen h3=3*(t^(3-1))
	* limit values of large hazards
		replace h2=. if h2>5
		replace h3=. if h3>5
	* label the hazards
	label var h_5 "p=0.5"
	label var h1 "p=1"
	label var h1_5 "p=1.5"
	label var h2 "p=2"
	label var h3 "p=3"
* produce the graph
	graph twoway (line h_5 t, lcolor(black) lpattern(solid)) ///
		(line h1 t, lcolor(black) lpattern(dash)) ///
		(line h1_5 t, lcolor(black) lpattern(shortdash)) ///
		(line h2 t, lcolor(black) lpattern(solid)) ///
		(line h3 t, lcolor(black) lpattern(dash)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("hazard", m(r+2)) xtitle("analysis time", m(t+2)) ///
		text(.3 8 "p=0.5", place(NE)) text(1.1 8 "p=1", place(NE)) ///
		text(3.4 6 "p=1.5", place(NE)) text(4 2.2 "p=2", place(NE)) ///
		text(4 0.5 "p=3", place(NE)) ylabel(,angle(horizontal))
	graph save "c:\ver2\ch19\figs\fig19_weib_haz", replace
	graph export "c:\ver2\ch19\figs\fig19_weib_haz.wmf", replace


*************************************************************************
* Figure 19.11 - Generate graphs showing proportional hazards
*************************************************************************
* open hypothetical dataset
	use fig19_cox_hr.dta, clear
* generate graph of Betas
	twoway line beta_0 beta_1 time, scheme(s1mono) lcolor(black black) ///
		lpattern(solid dash) lwidth(medium medium) legend(off) ///
		title("Log Hazard (Beta=0.69)") ///
		ytitle("log hazard", m(r+2) size(medium)) ///
		xtitle("time", m(t+2) size(medium)) ylabel(,angle(horizontal))   ///
		 plotregion(style(none)) saving(fig19_cox_hr1, replace)
* generate graph of Hazards
	twoway line haz_0 haz_1 time, scheme(s1mono) lcolor(black black) ///
		lpattern(solid dash) lwidth(medium medium) legend(off) ///
		title("Hazard (HR=2)") ///
		ytitle("hazard", m(r+2) size(medium)) ///
		xtitle("time", m(t+2) size(medium)) ylabel(,angle(horizontal)) ///
		 plotregion(style(none))  saving(fig19_cox_hr2, replace)
* combine the two graphs
	graph combine fig19_cox_hr2.gph fig19_cox_hr1.gph , ysize(4) xsize(8) scheme(s1mono)
	graph export "C:\ver2\ch19\figs\fig19_cox_hr.wmf", replace


*************************************************************************
* Example 19.5 Cox Proportional Hazrds Model
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* fit a Cox proportional Hazards model
	stset dar, f(preg)
	xi: stcox i.herd tx lact thin, nohr
* refit the model to obtain hazard ratios
	stcox
* evaluating methods of handling ties
	xi: stcox i.herd tx lact thin, nohr efron
	xi: stcox i.herd tx lact thin, nohr exactm
	xi: stcox i.herd tx lact thin, nohr exactp


*************************************************************************
* Figure 19.12 - K-M Survivor curves - Prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* obtain K-M survivor curves
	stset dar, f(preg)
	sts graph, by(tx) scheme(s1mono) legend(off) ///
		title("") ytitle("probability of not conceiving", m(r+2)) ///
		xtitle("time (days after onset of breeding period)", m(t+2)) ///
		 plotregion(style(none)) ylabel(,angle(horizontal))   ///
		 plot2opts(lcolor(black)) ylabel(,glwidth(medthick) glcolor(black) glpattern(dot)) ///
		text(.8 30 "no tx.", place(NE)) text(.6 10 "tx.", place(NE)) 
	graph save "C:\ver2\ch19\figs\fig19_km_prgl.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_km_prgl.wmf", replace


*************************************************************************
* Figure 19.13 - Baseline Hazard - Prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* obtain K-M survivor curves
	stset dar, f(preg)
* rescale lact so 0 = 1st lactation
	gen lact_1=lact-1
	xi: stcox i.herd tx lact_1 thin, nohr basehc(hc)
	stcurve, hazard scheme(s1mono) lcolor(black) ylabel(,angle(horizontal)) ///
		lpattern(solid) title("") plotregion(style(none))  ///
		ytitle("smoothed estimate of baseline hazard", m(r+2)) ///
		xtitle("time (days after start of breeding period)", m(t+2))		
	graph save "C:\ver2\ch19\figs\fig19__bh_prgl.gph", replace
	graph export "C:\ver2\ch19\figs\fig19__bh_prgl.wmf", replace


*************************************************************************
* Example 19.7 Stratified Cox Proportional Hazrds Model
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* fit a Cox proportional Hazards model - stratified by herd
	stset dar, f(preg)
	xi:stcox i.herd*tx lact thin, nohr strat(herd)
* evaluate the overall significance of the interaction term
	test _IherXtx_2 _IherXtx_3
* fit the same model herd as a fixed effect (not stratified)
	xi:stcox i.herd*tx lact thin, nohr 
* evaluate the overall significance of the interaction term
	test _IherXtx_2 _IherXtx_3


*************************************************************************
* Example 19.8 Time varying predictors in a Cox Proportional Hazrds Model
*************************************************************************
* open the ISA risk factor dataset
	use isa_risk.dta, clear
* compute a time variable for # of days since Apr. 1st
	gen days=date-apr01_97
	stset days , f(case) id(sitepen)
* compute the days to the first outbreak at each site
	gen days2=days if case==1
	sort site
	quietly by site: egen first_da=min(days2)
* list the data for 3 netpens before modification
	sort site netpen _d _t0
	list site netpen _t0 _t _d first_da if site==19 & netpen>=39 & netpen<=56, clean
* list the data after modification for a time varying predictor
	stsplit pos, after(time=first_da) at(0)
	replace pos=pos+1
	sort site netpen _d _t0
	list site netpen _t0 _t _d pos first_da if site==19 & netpen>=39 & netpen<=56, clean
	stcox pos


*************************************************************************
* Example 19.9 Aalen's linear hazards model
*************************************************************************
* open the prostaglandin trial dataset
	set more off
	use pgtrial.dta, clear
	stset dar, f(preg)
* Aalen's linear hazards model
	stlh tx , gen(cumbeta)
	line cumbetaB1 dar if dar<=50, plotregion(style(none))  ///
		 scheme(s1mono) lcolor(black black) lpattern(solid ) ///
		ytitle("Aalen's cumulative coef. for tx", m(r+2) size(med)) ///
		xtitle("time (days after start of breeding period)", m(t+2) size(med)) ///
		ylabel(,angle(horizontal)) 		
	graph save "C:\ver2\ch19\figs\fig19_aalen_lin_haz.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_aalen_lin_haz.wmf", replace


*************************************************************************
* Figure 19.16 Cumulative hazard plot - Prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg)
* fit a Cox proportional hazards model
	xi: stcox i.herd tx lact thin, nohr
	stphplot , by(tx) scheme(s1mono) title("") legend(off) ///
		plot1opts(lcolor(black) lpattern(solid) msymbol(o) mcolor(black)) ///
		plot2opts(lcolor(black) lpattern(dash) msymbol(+) mcolor(black)) ///
		ytitle("-ln cumulative hazard", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("ln(time)", m(t+2)) plotregion(style(none)) ///
		text(3 2 "tx = no", place(NE)) text(.8 1 "tx = yes", place(NE))
	graph save "C:\ver2\ch19\figs\fig19_phplot_prgl.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_phplot_prgl.wmf", replace


*************************************************************************
* Figure 19.17 - Cox vs KM plot - Prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* obtain K-M survivor curves
	stset dar, f(preg)
* fit a Cox proportional hazards model
	xi: stcox i.herd tx lact thin, nohr
	stcoxkm if _t<50, by(tx) scheme(s1mono) ///
		title("") legend(off) ytitle("survival probability", m(r+2)) ///
		xtitle("time", m(t+2)) plotregion(style(none)) ///
		text(.8 9 "KM: tx = no", place(NE)) text(.36 4  "KM: tx = yes", place(NE)) ///
		text(.5 9 "Cox", place(NE)) ylabel(,angle(horizontal)) ///
  		obs1opts(lcolor(black) lpattern(solid) msymbol(o) mcolor(black)) ///
    		obs2opts(lcolor(black) lpattern(solid) msymbol(d) mcolor(black))  ///
    		pred1opts(lcolor(black) lpattern(dash) msymbol(+) mcolor(black)) ///
    		pred2opts(lcolor(black) lpattern(dash) msymbol(x) mcolor(black))	
	graph save "C:\ver2\ch19\figs\fig19_kmplot_prgl.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_kmplot_prgl.wmf", replace


*************************************************************************
* Example 19.10 Time varying efects
*************************************************************************
* open the prostaglandin dataset
	use pgtrial.dta, clear
	stset dar, f(preg)
* fit a model with tretment as a tvc
	stcox tx, tvc(tx) texp(log(_t))


*************************************************************************
* Example  19.11 and Figure 19.18 - Schoenfeld residuals
*************************************************************************
* open the pgtrial dataset
	use pgtrial.dta, clear
* fit the Cox proportinal hazards model
* and save the Schoenfeld and scaled Schoenfeld residuals
	stset dar, f(preg) 
	xi: stcox i.herd tx lact thin, nohr sca(sca*) sch(sch*) 
* generate a smoothed graph of scaled Scheonfeld residuals vs log(time)
	estat phtest, log plot(lact) yline(0, lcolor(black) lpattern(dash))  ///
		ytitle("Scaled Schoenfeld - lactation", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("time (log scale)", m(t+2)) scheme(s1mono) ///
		xlab(1 5 20 50 100 300) mcolor(black) title("") plotregion(style(none)) 
	graph save "c:\ver2\ch19\figs\fig19_ph_schoen.gph", replace
	graph export "c:\ver2\ch19\figs\fig19_ph_schoen.wmf", replace
* compute test statistics for all predictors
	estat phtest, detail log


*************************************************************************
* Example  19.12 - Independence of Censoring
*************************************************************************
* open the pgtrial dataset
	set more off
	use pgtrial.dta, clear
* fit the Cox proportinal hazards model
* with -tx- as a time varying covariate
	stset dar, f(preg) 
	xi: stcox i.herd tx lact thin, nohr tvc(tx) texp(ln(_t)) 
* recode censoring to assume complete positive correlation
	preserve
	replace preg=1 if preg==0
	stset dar, f(preg) 
	xi: stcox i.herd tx lact thin, nohr tvc(tx) texp(ln(_t)) 
	restore
* recode censoring and dar to assume complete negative correlation
	preserve
	replace dar=400 if preg==0
	replace preg=1 if preg==0
	stset dar, f(preg) 
	xi: stcox i.herd tx lact thin, nohr tvc(tx) texp(ln(_t))   
	restore


*************************************************************************
* Example 19.13 and Figure 19.19 Evaluating overall fit of a model / computing r2
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* fit a Cox proportional Hazards model
	* note - based on a model without tx as a TVC
	stset dar, f(preg)
	xi: stcox i.herd tx lact thin, nohr mg(mgale)
* compute modified Cox-Snell residuals
	predict cs, csnell
	* replace cs=cs+0.693 if preg==0
		label var cs "Modified Cox-Snell residuals"
   * re "stset" the data and fit the new Cox model
		stset cs, f(preg)
		sts gen ch=na
		sort cs
		line ch cs cs,  xlab(0 1 2 3 4) ylab(0 1 2 3 4) scheme(s1mono) ///
			ytitle("Cumulative hazard",m(r+2)) legend(off) ylabel(,angle(horizontal)) ///
			plotregion(style(none)) 
		graph save "C:\ver2\ch19\figs\fig19_cs_fit.gph", replace
		graph export "C:\ver2\ch19\figs\fig19_cs_fit.wmf", replace
	* refit the line but limit it to cs<=1 (not shown in text)
		line ch cs cs if cs<=1,  xlab(0 (0.2) 1) ylab(0 (0.2) 1) scheme(s1mono) ///
			ytitle("Cumulative hazard",m(r+2)) legend(off) ylabel(,angle(horizontal)) ///
			plotregion(style(none)) 
		graph save "C:\ver2\ch19\figs\fig19_cs_fit2.gph", replace
		graph export "C:\ver2\ch19\figs\fig19_cs_fit2.wmf", replace
* Goodness-of-fit tests 
	* open the prostaglandin trial dataset
		set more off
		use "C:\ver2\data\pgtrial.dta", clear
	* fit a Cox proportional Hazards model
		stset dar, f(preg)
		xi: stcox i.herd tx lact thin, nohr mg(mgale)
		stcoxgof			/* Gronnesby and Borgan omnibus test */
		stcoxgof, mol(5)		/* Moreau, O'Quigley and Lellouch ominbus test */
		stcoxgof, mom(5)		/* Moreau, O'Quigley and Mesbah test for proportional hazards */ 
* Concordance
	* open the prostaglandin trial dataset
		set more off
		use "C:\ver2\data\pgtrial.dta", clear
	* fit a Cox proportional Hazards model
		stset dar, f(preg)
		xi: stcox i.herd tx lact thin, nohr mg(mgale)
		estat concordance
* computing r2
	* open the prostaglandin trial dataset
		use pgtrial.dta, clear
		stset dar, f(preg)
		xi: str2ph stcox i.herd tx lact thin, nohr bo(5000)


*************************************************************************
* Example 19.14 and Figure 19.20 Evaluating functional form of a predictor
*************************************************************************
* open the prostaglandin trial dataset
	set more off
	use pgtrial.dta, clear
* fit a Cox proportional Hazards model (without -lact-)
	* note - based on a model without tx as a TVC
	capture drop mg*
	stset dar, f(preg)
	xi: stcox i.herd tx thin, nohr mgale(mg)
	lowess mg lact , scheme(s1mono) xtitle("lactation", m(t+2)) ///
		mcolor(black) lcolor(black) plotregion(style(none)) ylabel(,angle(horizontal)) ///
		ytitle("Martingale residual",m(r+2)) legend(off) title("")
	graph save "C:\ver2\ch19\figs\fig19_mgale.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_mgale.wmf", replace
* including a quadratic term for lact
	gen lact_ct=lact-2.5
	gen lact_ct_sq=lact_ct^2
	capture drop mg*
	xi: stcox i.herd tx thin lact_ct lact_ct_sq, nohr


*************************************************************************
* Figure 19.21 - Detecting outliers (deviance residuals)
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg)
* detecting outliers - deviance residuals
	* note - based on a model without tx as a TVC
	gen lact_ct=lact-2.5
	gen lact_ct_sq=lact_ct^2
	xi: stcox tx i.herd lact_ct lact_ct_sq , nohr mgale(mg) 
		predict dev, dev
	scatter dev _t , scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("deviance residual",m(r+2)) plotregion(style(none))  ///
		text(2.8 8 "Cows 1,2,3,78,79,80", place(E)) ylabel(,angle(horizontal)) 
	graph save "C:\ver2\ch19\figs\fig19_dev_res.gph", replace
	graph export "C:\ver2\ch19\figs\fig19_dev_res.wmf", replace
* redraw the graph with only residuals >2 (not in text)
	scatter dev _t if dev>2, mlabel(cow) scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("deviance residual",m(r+2)) plotregion(style(none)) ///
		ylabel(,angle(horizontal)) 


*************************************************************************
* Figure 19.22 - Detecting influential points - Score residuals
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg)
	* note - based on a model without tx as a TVC
		gen lact_ct=lact-2.5
		gen lact_ct_sq=lact_ct^2
		capture drop mg 
		capture drop esr*
	xi: stcox tx i.herd lact_ct lact_ct_sq ,  esr(esr*) mg(mg)
		* plot score residuals
		scatter esr1 _t , mlabel(cow) scheme(s1mono) xtitle("time (days)", m(t+2)) ///
			mcolor(black) ytitle("score residual",m(r+2)) plotregion(style(none)) ///
			ylabel(,angle(horizontal))  
			graph save "C:\ver2\ch19\figs\fig19_score_res.gph", replace
			graph export "C:\ver2\ch19\figs\fig19_score_res.wmf", replace


*************************************************************************
* Figure 19.23 Detecting influential points - Scaled score residuals
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg)
	* detecting influential points - scaled score residuals
	* note - based on a model without tx as a TVC
		gen lact_ct=lact-2.5
		gen lact_ct_sq=lact_ct^2
		capture drop mg 
		capture drop esr*
		capture drop ssr*
	xi: stcox tx i.herd lact_ct lact_ct_sq ,  esr(esr*) mg(mg)
	* compute scaled score residuals
		mkmat esr1 esr2 esr3 esr4 esr5, matrix(esr)
		mat V=e(V)
		mat ssr=esr*V
		svmat ssr, names(ssr)
		label var ssr1 "ssr - treatment"
		scatter ssr1 _t , mlabel(cow) scheme(s1mono) xtitle("time (days)", m(t+2)) ///
			mcolor(black) ytitle("scaled score residual", m(r-6)) plotregion(style(none)) ///
			ylabel(,angle(horizontal)) 
			graph save "C:\ver2\ch19\figs\fig19_scal_score_res.gph", replace
			graph export "C:\ver2\ch19\figs\fig19_scal_score_res.wmf", replace
	* refit model without cow 76
		xi: stcox tx i.herd lact_ct lact_ct_sq  if cow~=76


*************************************************************************
* Example 19.15 - Exponential regression
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg) id(cow)
* fit an exponential model after rescaling -lact- 
	gen lact1=lact-1
	xi: streg i.herd tx lact1 thin, nohr dist(exp)


*************************************************************************
* Example 19.16 and Figure 19.24 - Exponential regression - piecewise constant baseline hazard
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
	stset dar, f(preg) id(cow)
* derive a step function estimate of the baseline hazard
	sum _t
	stsplit day, at(0(20)346)
		gen day0_20= 0<=day & day<20
		gen day21_40= 20<=day & day<40
		gen day41_80= 40<=day & day<80
		gen day81_120= 80<=day & day<120
		gen day121_= 120<=day
* fit an exponential model with a piecewise constant hazard
	gen lact1=lact-1
	xi: streg day21_40 day41_80 day81_120 day121_ i.herd tx lact1 thin, nohr dist(exp)
* compute the baseline hazard
	gen h0=exp(_b[_cons] + _b[day21_40]*day21_40 + _b[day41_80]*day41_80 + ///
		_b[day81_120]*day81_120 + _b[day121_]*day121_)
* graph the baseline hazard (up to day 200)
	sort day
	twoway line h0 day if day<=200, connect(J) scheme(s1mono) ///
		ytitle("baseline hazard",m(r+2)) xtitle("time (days)", m(t+2)) ///
		plotregion(style(none)) ylabel(,angle(horizontal)) 
	graph save "c:\ver2\ch19\figs\fig19_exp_reg_pc.gph", replace
	graph export "c:\ver2\ch19\figs\fig19_exp_reg_pc.wmf", replace


*************************************************************************
* Example 19.17  Weibull model 
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* rescale the lactation variable
	gen lact1=lact-1
* fit a Weibull model (no time varing covariates)
	stset dar, f(preg) id(cow)
	xi: streg i.herd tx lact1 thin, nohr dist(weib)
		estimates store weibull
* modify the model
* making -tx- a TVC with log(time)
	* results not shown in book
	stsplit day, at(1(1)346)
	stset dar, f(preg) id(cow)
	gen tx_day=tx*day
	gen tx_lnday=tx*(log(day))
	xi: streg i.herd tx lact1 thin tx_lnday, dist(weib) tr


*************************************************************************
* Fig 19.25 - Hazard function and survival time distributions - log logitic model
*************************************************************************
* set up the initial dataset of 200 records
	clear
	set obs 2000
	set seed 12345
	set more off
	gen id=_n
* generate values for hazard over 100 days
	gen t=id if id<=100
	gen x1=0
	gen day=.
	gen fail=0
	gen failday=.
	gen hz=.
* compute the baseline log-logistic hazard (three functions)
	* formulae derived from Collett 2nd Ed (p 176, 195-200)
  	 	gen k=. 		/* 1/gamma in Stata */
		gen th=. 		/* theta - a function of B0 in Stata */
		* k=4, th=-11.98    	k=4, th=-11.98 --> median t is 20
  		 	replace k=4	
			replace th=-11.98
			gen hz_4=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=2, th=-5.99    	k=2, th=-5.99 --> median t is 20
  		 	replace k=2	
			replace th=-5.99
			gen hz_2=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=1, th=-3    	k=1, th=-3 --> median t is 20
  		 	replace k=1	
			replace th=-3
			gen hz_1=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=0.5, th=-1.5    	k=0.5, th=-1.5 --> median t is 20
  		 	replace k=0.5	
			replace th=-1.5
			gen hz_05=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		line hz_4 hz_2 hz_05 t, scheme(s1mono) xtitle("time (days)", m(t+2)) ///
			lcolor(black black black) lpattern(solid dash dot) ///
			clwidth(medium medium thick) legend(off) ///
			ytitle("hazard",m(r+2)) plotregion(style(none)) ///
			text(.1 40 "gamma=0.25", place(NE)) text(.05 30 "gamma=0.5", place(NE)) ///
			text(.01 40 "gamma=2", place(NE)) ylabel(,angle(horizontal))
		graph save "C:\ver2\ch19\data\fig19_log_logist_a.gph", replace
* simulate the data, one day at a time
	local i = 1
	while `i' <= 100 {			
		quietly replace day=`i'		
     		display day
	* obtain the baseline hazard appropriate for the day
     		quietly replace hz=hz_4[day]
	* generate a uniform random number
  		gen rand=uniform()
	* determine if the individual failed
  		quietly replace failday=`i' if rand<hz & fail==0
  		quietly replace fail=1 if rand<hz & fail==0
	* update the counter and repeat loop
  		drop rand
  	local i = `i' + 1 
	}
* censor all remaining observations at day 101 and graph distn. of survival times
	replace failday=101 if fail==0
      stset failday, f(fail) id(id)
	histogram failday, width(2) start(0) frequency ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("frequency",m(r+2)) plotregion(style(none)) ///
		text(100 50 "gamma=0.25", place(NE)) ylabel(,angle(horizontal))
	graph save "C:\ver2\ch19\data\fig19_log_logist_b.gph", replace
* combine the two graphs
	graph combine fig19_log_logist_a.gph fig19_log_logist_b.gph, ///
		scheme(s1mono) plotregion(style(none)) 
	graph export "C:\ver2\ch19\figs\fig19_log_logist.wmf", replace


*************************************************************************
* Example 19.18 - Log-logistic model 
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* rescale the lactation variable
	gen lact1=lact-1
* fit a log-logistic model (no time varing covariates)
	stset dar, f(preg) id(cow)
	xi: streg i.herd tx lact1 thin, dist(llogistic)


*************************************************************************
* Example 19.19 - Comparison of parametric models
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* rescale the lactation variable
	gen lact1=lact-1
* fit a Weibull model (no time varing covariates)
	stset dar, f(preg) id(cow)
	xi: streg i.herd tx lact1 thin, dist(exp) tr
		estat ic
	xi: streg i.herd tx lact1 thin, dist(weibull) tr
		estat ic
	xi: streg i.herd tx lact1 thin, dist(llogistic) tr
		estat ic
	xi: streg i.herd tx lact1 thin, dist(lnormal) tr
		estat ic
	xi: streg i.herd tx lact1 thin, dist(gamma) tr
		estat ic
	xi: streg i.herd tx lact1 thin, dist(gompertz)
		estat ic


*************************************************************************
* Fig 19.26 - Effects of individual frailties
*************************************************************************
* set up the initial dataset of 2000 records
	clear
	set obs 2000
	set seed 12345
	set more off
	gen id=_n
* set up some variables
	gen t=id if id<=100
	gen x1=0
	gen day=.
	gen fail=0
	gen failday=.
*  generate gamma frailties (xg) - mu=1 sd=1 
        gen temp=1
        rndgamx temp, s(1)
			sum xg
			histogram xg, start(0) width(.5)
* compute the baseline constant hazard (0.1) and add frailty effect
	gen hz=0.05 * xg 
* simulate the data, one day at a time
	local i = 1
   	while `i' <= 100 {
   		quietly replace day=`i'
     		display day
	* generate a uniform random number
  		gen rand=uniform()
	* determine if the individual failed
  		quietly replace failday=`i' if rand<hz & fail==0
  		quietly replace fail=1 if rand<hz & fail==0
	* update the counter and repeat loop
  		drop rand
  	local i = `i' + 1 
	}
* censor all remaining observations at day 101
	replace failday=101 if fail==0
      stset failday, f(fail)
* fit a Weibull model (no predictors)
	streg , dist(weibull) nohr
	stcurve, hazard ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("hazard",m(r+2)) plotregion(style(none)) ///
		ylabel(,angle(horizontal)) title("Empirical hazard")
	graph export "C:\ver2\ch19\figs\fig19_ind_frailty.wmf", replace
* fit a Weibull model with a gamma frailty (not in text)
	streg , dist(weibull) frailty(gamma)
	stcurve, hazard alpha1  ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("hazard",m(r+2)) plotregion(style(none)) ///
		ylabel(,angle(horizontal)) title("Conditional hazard")

*************************************************************************
* Example 19.20  Individual frailty - prostaglandin data
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* rescale the lactation variable
	gen lact1=lact-1
* fit a Weibull model (no time varing covariates)
	stset dar, f(preg) id(cow)
	xi: streg i.herd tx lact1 thin, nohr dist(weib) fr(gamma)


*************************************************************************
* Example 19.21 - Weibull model with shared gamma frailty 
*************************************************************************
* open the culling dataset
	use culling.dta, clear
* fit a Weibull model 
	stset dar, f(culled) id(cow)
	xi: streg i.lact_c3 johnes, nohr dist(weib) frailty(gamma) shared(herd)


*************************************************************************
* Example 19.22 - Cox model with shared gamma frailty 
*************************************************************************
* open the culling dataset
	use culling.dta, clear
* fit a Cox model 
	stset dar, f(culled) id(cow)
	xi: stcox i.lact_c3 johnes, nohr frailty(gamma) shared(herd)


*************************************************************************
* Example 19.23 - Cox Model (shared frailty)  - fit by Poisson regression
*************************************************************************
* open the culling dataset
	use culling.dta, clear
	stset dar, f(culled) id(cow)
* fit ordinary and shared frailty Cox models
	xi: stcox i.lact_c3 johnes, nohr
	xi: stcox i.lact_c3 johnes, nohr shared(herd) /* gamma distn for frailties */
* gernerate events dataset
	stsplit, at(failures) riskset(RS)
	replace culled =0 if culled==.
	sort cow dar
	by cow: gen len=cond(_n>1, dar-dar[_n-1], dar)
	gen lnlen=ln(len)
* ordinary Cox model via Poisson
	* fixed effects for risk intervals
		xi: xtpoisson culled i.lact_c3 johnes, fe i(RS) offset(lnlen)
	* replace fixed effects with smoothed high-order function of time
		orthpoly dar, gen(t1-t4) degree(4)
		xi: poisson culled t1-t4 i.lact_c3 johnes, offset(lnlen)
* random effects model (shared frailty)
	* gamma frailty
		xi: xtpoisson culled t1-t4 i.lact_c3 johnes, re i(herd) offset(lnlen)
	* log-normal frailty (not in text)
		xi: xtpoisson culled t1-t4 i.lact_c3 johnes, re i(herd) offset(lnlen) normal


*************************************************************************
* Example 19.24 - Multiple failure time data
*************************************************************************
* open the clinical mastitis dataset
	use clin_mast.dta, clear
* structure data for Anderson-gill model
	* generate number of previous cases
		gen prev_cases=0
		sort cowid d
		quietly by cowid: replace prev_cases=(_n-1) if _n>0 
	stset d, f(mast) id(cowid) exit(mast==0)
	* fit a Cox model with robust SE
		stcox pr prev_cases, nohr robust
		* adding a quadratic for pr
		gen pr_sq=pr^2
		stcox pr pr_sq prev_cases, nohr robust
			/* marginal evidence of quadratic effect - results not shown in the book */
	* fit a Weibull model with robust SE
		streg pr prev_cases, dist(weibull) nohr robust
	* fit a Weibull model with a gamma frailty
		streg pr prev_cases, dist(weibull) nohr shared(cowid)
	* create some variables for printing
		gen time0_ag=_t0
* structure data for PWP (conditional risk set) model
	* generate identifier for risk set
		gen risk_set=prev_cases+1
		sort cowid d
	* generate time zero = 0 and length of interval
		gen time0_pwp=0
		gen d2=d
		quietly by cowid: replace d2=d2[_n]-d2[_n-1] if _n>1
	stset d2, f(mast) id(cowid) enter(time0_pwp) exit(mast==0)
	* fit a Cox model with robust SE
		stcox pr , nohr robust strata(risk_set)
	* fit a Weibull model with robust SE	
		streg pr , dist(weibull) strata(risk_set) nohr robust
* listing data for the example
	list herd cowid pr prev_cases mast time0_ag d time0_pwp d2 risk_set if cowid<=15, ///
		abb(6) clean noobs


*************************************************************************
* Example 19.25 - Discrete time survival analysis
*************************************************************************
* open the prostaglandin trial dataset
	use pgtrial.dta, clear
* divide data into 15/30/60 day intervals
	* cutpoints included in upper, censored observations excluded
		gen mdar=dar+0.0001				/* add a small quantity to -dar- */
		stset mdar, f(preg) id(cow)
		stsplit per, at(0 15 30 45 60 90 120 180 240 300)
			* drop last period for censored observations (ie cens. obs. excluded)
				bysort cow: drop if _n==_N & _n>1 & preg==0
					tab per _d if per<=45, row
	* fit a logistic model to the data
	xi:logit _d i.per tx i.herd lact thin
		* include treatment by time interval interactions
			xi:logit _d i.per*tx i.herd lact thin
	* complementary log-log regression
			xi:cloglog _d i.per*tx i.herd lact thin
* divide data into 0-2, 3-4 5-23 24-27, 28-59 ... day intervals
	* cutpoints included in upper, censored observations excluded
	use pgtrial.dta, clear
		gen mdar=dar+0.0001				/* add a small quantity to -dar- */
		stset mdar, f(preg) id(cow)
		stsplit per, at(0 3 5 24 28 60 90 120 180 240 300)
			* drop last period for censored observations (ie cens. obs. excluded)
				bysort cow: drop if _n==_N & _n>1 & preg==0
					tab per _d, row
	* fit a logistic model to the data
	xi:logit _d i.per tx i.herd lact thin
		* include treatment by time interval interactions
			xi:logit _d i.per*tx i.herd lact thin
				testparm  _IperXtx_3- _IperXtx_300

*************************************************************************
* Figure 19.27 - Complementary log-log function
*************************************************************************
	clear
* create probability scale
	set obs 100
	gen p=(_n-1)/100
* compute logit
	gen logit=log(p/(1-p))
* compute comp. log-log
	gen cloglog=log(-log(1-p))
* produce the graph
	twoway (line logit p, lpattern(dash) msymbol(none) lcolor(black)) ///
		(line cloglog p, lpattern(solid) msymbol(none) lcolor(black)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("logit / cloglog", m(r+2)) ///
		xtitle("probability", m(t+2)) ylabel(,angle(horizontal)) ///
		text(2 .75 "logit", place(NE)) text(-1 .5  "complementary log-log", place(NE))
	graph export "c:\ver2\ch19\figs\fig19_cloglog.wmf", replace


*************************************************************************
* Example 19.26 - Complementary log-log analysis of prostaglandin data
*************************************************************************
	use pgtrial.dta, clear
		gen mdar=dar+0.0001				/* add a small quantity to -dar- */
		stset mdar, f(preg) id(cow)
		stsplit per, at(0 3 5 24 28 60 90 120 180 240 300)
			* drop last period for censored observations (ie cens. obs. excluded)
				bysort cow: drop if _n==_N & _n>1 & preg==0
					tab per _d, row
	* fit a cloglog model to the data
		xi:cloglog _d i.per*tx i.herd lact thin


*************************************************************************
* Example 19.27 Sample size calculation
*************************************************************************
	* Scenario a
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) recrt(0 0, 1, 0 ) ///
			distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario b
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) lg(1 2 3 ) ///
			ldf( .05 .15 .3 .4, 1 2 3 4; .05 .15 .3 .4, 1 2 3 4; .05 .15 .3 .4, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario c
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) lg(1 2 3 ) ///
			ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario d
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) wg(1 2 ) ///
			wdf(.2, 1; .1, 1) crover(3 3 ) lg(1 2 3 ) ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario e
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(3 2 ) wg(1 2 ) wdf(.2, 1; .1, 1) ///
			crover(3 3 ) lg(1 2 3 ) ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
