* ch20_all.do
* Introduction to clustered data - all calculations
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch20\data		/* this line is computer dependent */
	capture log close ch20_all
	log using ch20_all, name(ch20_all) replace 

*************************************************************************
* Example 20.1 - Hierarchical structure of reu_cfs data
*************************************************************************
use reu_cfs, clear
   * determine number of lowest level units
		codebook obs
   * aggregate to cow level
		collapse (count) npercow=lncfs, by(region herd cow)
   * determine distribution at cow level
		sum npercow
   * aggregate to herd level
		collapse (count) nperherd=npercow, by(region herd)
   * determine distribution at herd level
		sum nperherd
   * aggregate to region level
		collapse (count) nperreg=nperherd, by(region)
   * determine distribution at region level
		sum nperreg

*************************************************************************
* Example 20.2 - clustering in a continuous data model
*************************************************************************
* open the simulated data with -X- as a herd level variable
	use simcont_clustherd.dta, clear
	* fit an ordinary linear regression
		regress milk X
	* fit a linear mixed model with herd as the cluster
		xtmixed milk X || herd:
	* collapse the data to the herd level and fit an ordinary reg.
		collapse (mean) milk X, by(herd)
		regress milk X
* open the simulated data with -X- as a cow level variable
	use simcont_clustcow.dta, clear
	* fit an ordinary linear regression
		regress milk X
	* fit a linear mixed model with herd as the cluster
		xtmixed milk X || herd:

*************************************************************************
* Example 20.3 - clustering in a binary data model
*************************************************************************
* open the dataset -X- as a herd level variable
	use simbin_clustherd.dta, clear
	* fit an ordinary logistic regression
		logit Y X
	* fit a random effects model with herd as the cluster
		xtlogit Y X, i(herd)
* open the dataset -X- as a cow level variable
	use simbin_clustcow.dta, clear
	* fit an ordinary logistic regression
		logit Y X
	* fit a a random effects model with herd as the cluster
		xtlogit Y X, i(herd)

*************************************************************************
* Figure 20.5 - Simulation study on effects of clustering
*************************************************************************
* open the dataset with 1000 simulation results
	use simbin_1000.dta, clear
	* graph the coef. for X and its SE over various levels of ICC of X 
	twoway (line xbias_b1 icc, sort lcolor(black) lpattern(solid)) ///
		(line xbias_se1 icc, lcolor(black) lpattern(dash)), ///
		ytitle("bias", m(r+2)) ylabel(#6, angle(horizontal)) xtitle("ICC of X (%)", m(t+2)) ///
		legend(off) name(simbin_1, replace) scheme(s1mono) ///
		text(.6 65 "SE(beta)", place(NE)) text(.87 50  "beta", place(NE)) 
* open the dataset with the raw data from the 1000 simulations
	use sim/simbin_1000_all.dta, clear
	replace icc=100*icc
	* graph the SD of the estimates from ordinary and random effects log. regn.
		preserve
		collapse (mean) o_mu=X1o_b1b r_mu=X1r_b1b (sd) o_sd=X1o_b1b r_sd=X1r_b1b, by(icc)
		twoway (line o_sd icc, sort lcolor(black) lpattern(solid)) ///
			(line r_sd icc, lcolor(black) lpattern(dash)), ///
			ytitle("SD of estimates of beta", m(r+2)) ///
			ylabel(#6, angle(horizontal)) xtitle("ICC of X (%)", m(t+2)) ///
			legend(off) name(simbin_2, replace) scheme(s1mono) ///
			text(.09 30 "ordinary" "logistic regression", place(NE)) ///
			text(.05 58 "random effects" "logistic regression", place(NE))
		restore
* combine the two graphs
	graph combine simbin_1 simbin_2, scheme(s1mono)  ysize(4) xsize(8) 
	graph save "figs/fig19_kmplot_prgl.gph", replace
	graph export "figs/fig19_kmplot_prgl.wmf", replace

*************************************************************************
* Figure 20.6 - Clustering and confounding
*************************************************************************
* open the dataset with 1000 simulation results
	use simbin_1000.dta, clear
	* graph the coef. for X and its SE over various levels of ICC of Z 
	twoway (line zbias_b1 icc, sort lcolor(black) lpattern(solid)) ///
		(line zbias_se1 icc, lcolor(black) lpattern(dash)), ///
		ytitle("bias", m(r+2)) ylabel(#6, angle(horizontal)) xtitle("ICC of Z (%)", m(t+2)) ///
		legend(off) name(Z_B1_dich, replace) scheme(s1mono) ///
		text(1.37 20 "Beta", place(NE)) text(.97 50  "SE(Beta)", place(NE)) 
	graph save "figs/fig20_conf.gph", replace
	graph export "figs/fig20_conf.wmf", replace

*************************************************************************
* Example 20.4 - Analyses for binary data
*************************************************************************
* open the simulated binary data with -X- as a cow-level variable
	use simbin_clustcow.dta, clear
	* fit a fixed effects logistic regression model
		xi:logit Y X i.herd
	* carry out a stratified analysis
		cc Y X, by(herd)
	* manual calculation of SE on log-scale
		di (r(ub_or)-r(lb_or))/2/(2*1.96)
	* logistic regression with robust SE
		logit Y X, vce(cluster herd)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee Y X, fam(bin) link(logit) i(herd) corr(exch) robust
	* survey methods with clustering at herds
		svyset herd
		svy: logit Y X

* open the simulated binary data with -X- as a herd-level variable
	use simbin_clustherd.dta, clear
	* aggregate to herd level
		collapse X (sum) pos=Y (count)n=Y, by(herd)
	* fit a generalised linear model with overdispersion
		glm pos X, fam(bin n) link(logit) scale(x2)
	* Williams method (williams command from J Hilbe)
		williams pos n X
	use simbin_clustherd.dta, clear
	* logistic regression with robust SE
		logit Y X, vce(cluster herd)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee Y X, fam(bin) link(logit) i(herd) corr(exch) robust
	* survey methods with clustering at herds
		svyset herd
		svy: logit Y X


*************************************************************************
* Example 20.5 - Analyses for continuous data
*************************************************************************
* open the simulated continuous data with -X- as a cow-level variable
	use simcont_clustcow.dta, clear
	* fit a fixed effects linear regression model
		xi: regress milk X i.herd
	* linear regression with robust SE
		regress milk X, vce(cluster herd)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee milk X, i(herd) corr(exch) robust
	* survey methods with clustering at herds
		svyset herd
		svy: regress milk X

* open the simulated continuous data with -X- as a herd-level variable
	use simcont_clustherd.dta, clear
	* linear regression with robust SE
		regress milk X, vce(cluster herd)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee milk X, i(herd) corr(exch) robust
	* survey methods with clustering at herds
		svyset herd
		svy: regress milk X


*************************************************************************
* Example 20.6 - Survey analysis for complex survey data
*************************************************************************
* open the survey sampling dataset from Chapter 2
	use dairy_dis.dta, clear
	* create a herd-level variable (propn. pos to leukosis)
		bysort herd: egen l_pos=sum(leukosis)
		bysort herd: egen l_n=count(leukosis)
		gen leuk_hrd=l_pos/l_n
		codebook leuk_hrd
	* survey methods: herd clustering (only)
		svyset herd, clear 
		svy: logit neospora lact leuk_hrd
	* same results with robust variance
		logit neospora lact leuk_hrd, vce(cluster herd)
	* survey methods: herd clustering and sampling weights
		svyset herd [pw=weight], clear
		svy: logit neospora lact leuk_hrd
	* same results with robust variance
		logit neospora lact leuk_hrd [pw=weight], vce(cluster herd)
	* survey methods: herd clustering, sampling weights and fixed province effects
		svyset herd [pw=weight], clear
		xi: svy: logit neospora lact leuk_hrd i.prov
	* same results with robust variance
		xi: logit neospora lact leuk_hrd i.prov [pw=weight], vce(cluster herd)
	* survey methods: herd clustering, sampling weights and stratification by province
		svyset herd [pw=weight], strata(prov) clear
		svy: logit neospora lact leuk_hrd
	* survey methods: herd clustering, sampling weights, stratification by province and fpc at herd and cow levels
		svyset herd [pw=weight], strata(prov) fpc(tot_hrd) || cow , fpc(tot_cow)
		svy: logit neospora lact leuk_hrd  
