* Opening boilerplate

* ch21_all.do - Mixed models for continuous data
	* command for all examples and figures in Chapter 21
	* and a few analyses that are not included in the chapter
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch21\data		/* this line is computer dependent */
	capture log close ch21_all
	log using ch21_all, name(ch21_all) replace 

*************************************************************************
* Example 21.1 - Variance components and random effects
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
* estimate parameters in 'null' model
	xtmixed lnscc || herdid:, var

*************************************************************************
* Example 21.2 - Clustering in a continuous data model
*************************************************************************
* standardize hsize and dim by subtracting the approx median and scaling by 100
	capture drop shsize sdim
	gen sdim=(dim-150)/100
	gen shsize=(hsize-45)/100
	label var sdim "stand.dim: (dim-150)/100"
	label var shsize "stand.hsize: (hsize-45)/100"
* estimate parameters with all predictors
	xi: xtmixed lnscc shsize heifer i.season sdim || herdid:, var

*************************************************************************
* Example 21.3 - Intra-class correlations in a 4-level mixed model
*************************************************************************
* open the reucfs dataset
	use reucfs, clear
* fit 4-level model with the two predictors (ai and heifer) in the dataset
	xtmixed lncfs ai heifer || region: || herd: || cow:, var

*************************************************************************
* Fig 21.1 - Interaction graph
*************************************************************************
twoway (function y=3 + .004*x, range(0 250) lcolor(black)) (function y=3.3 + .004*x, range(0 250) lcolor(black)) ///
	(function y=3.5 + .004*x, range(0 250) lcolor(black)) (function y=3.6 + .004*x, range(0 250) lcolor(black)) ///
	(function y=4 + .004*x, range(0 250) lcolor(black)), scheme(s1mono)  ///
	ytitle("lnscc(y)", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("days in milk (x)", m(t+2)) plotregion(style(none)) legend(off) ///
	title("additive model (no interaction") 
	graph save "figs/fig21_intn_1.gph", replace
	graph export "figs/fig21_intn_1.wmf", replace

twoway (function y=3 + .0045*x, range(0 250) lcolor(black)) (function y=3.3 + .005*x, range(0 250) lcolor(black)) ///
	(function y=3.5 + .001*x, range(0 250) lcolor(black)) (function y=3.6 + .003*x, range(0 250) lcolor(black)) ///
	(function y=4 + .002*x, range(0 250) lcolor(black)), scheme(s1mono)  ///
	ytitle("lnscc(y)", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("days in milk (x)", m(t+2)) plotregion(style(none)) legend(off) ///
	title("interaction - parity and days in milk") 
	graph save "figs/fig21_intn_2.gph", replace
	graph export "figs/fig21_intn_2.wmf", replace

* combine the two graphs
	graph combine "figs/fig21_intn_1.gph" "figs/fig21_intn_2.gph", ///
	 ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig21_intn_comb.gph", replace
	graph export "figs/fig21_intn_comb.wmf", replace

*************************************************************************
* Example 21.4 - Random slopes for dim
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
* estimate random slope parameters for sdim
	xi: xtmixed lnscc shsize heifer i.season sdim || herdid: sdim, cov(un) var

*************************************************************************
* Figure 21.2 - Variance function of random slopes model
*************************************************************************
* generate pairs of points (dim,var(y)) based on Eq 21.11
	clear
	set obs 200
	gen dim=_n
	gen sdim=(dim-150)/100
	gen vary=0.1297069+sdim^2*0.0425701+2*sdim*0.0052683+1.541299
	* plot points connected
		twoway (connected vary dim, sort msymbol(none) lwidth(thick)) ///
			, ytitle(var(Y), m(r+2)) ylabel(1.6(0.1)1.8, angle(horizontal)) ///
			xtitle(days, m(t+2)) xlabel(0(100)200) ///
			scheme(s1mono) 
	graph save "figs/fig21_varfct.gph", replace
	graph export "figs/fig21_varfct.wmf", replace

*************************************************************************
* Example 21.5 - Random slopes for dim
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
* estimate random slope parameters for dim
	xi: xtmixed lnscc shsize heifer i.season sdim || herdid: heifer, cov(un) var

*************************************************************************
* Figure 21.3 - Contextual effects
*************************************************************************
use cntx.dta, clear
twoway (scatter mid1 x) (function y=0.5852 + .0557*x, range(1 9) lpat(dash) lcol(black)) ///
	(function y=0.8 - 3*.0557 + .0557*x, range(1 5) lcolor(black)) ///
	(function y=0.7 - 4*.0557 + .0557*x, range(2 6)lcolor(black)) ///
	(function y=1.0 - 5*.0557 + .0557*x, range(3 7)lcolor(black)) ///
	(function y=0.75 - 6*.0557 + .0557*x, range(4 8)lcolor(black))  ///
	(function y=1.1 - 7*.0557 + .0557*x, range(5 9)lcolor(black))  ///
	(function y=1.0 - 8*.0557 + .0557*x, range(6 10)lcolor(black))  ///
	, scheme(s1mono) 	ytitle("y", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("x", m(t+2)) plotregion(style(none)) legend(off) ///
	title("no contextual effect") 
	graph save "figs/fig21_cntx_1.gph", replace
	graph export "figs/fig21_cntx_1.wmf", replace
twoway (scatter mid2 x) (function y=1.1981 - .0557*x, range(1 9) lpat(dash) lcol(black)) ///
	(function y=1.0 - 3*.0557 + .0557*x, range(1 5) lcolor(black)) ///
	(function y=1.1 - 4*.0557 + .0557*x, range(2 6)lcolor(black)) ///
	(function y=0.75 - 5*.0557 + .0557*x, range(3 7)lcolor(black)) ///
	(function y=1.0 - 6*.0557 + .0557*x, range(4 8)lcolor(black))  ///
	(function y=0.7 - 7*.0557 + .0557*x, range(5 9)lcolor(black))  ///
	(function y=0.8 - 8*.0557 + .0557*x, range(6 10)lcolor(black))  ///
	, scheme(s1mono) 	ytitle("y", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("x", m(t+2)) plotregion(style(none)) legend(off) ///
	title("contextual effect") 
	graph save "figs/fig21_cntx_2.gph", replace
	graph export "figs/fig21_cntx_2.wmf", replace
* combine the two graphs
	graph combine "figs/fig21_cntx_1.gph" "figs/fig21_cntx_2.gph", ///
	 ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig21_cntx_comb.gph", replace
	graph export "figs/fig21_cntx_comb.wmf", replace

*************************************************************************
* Example 21.6 - Contextual effects
*************************************************************************
* open the scc_heifer dataset
	use scc_heifer, clear
   * compute herd means of predictor (lnsccel)
		bysort herd: egen hlnsccel=mean(lnsccel)
   * compute group-mean centered version of lnsccel
		gen zlnsccel=lnsccel-hlnsccel
   * estimate random intercept model
		xi: xtmixed lnscc lnsccel || herd:, var
   * estimate contextual model (two versions)
		xi: xtmixed lnscc lnsccel hlnsccel || herd:, var
		xi: xtmixed lnscc zlnsccel hlnsccel || herd:, var
   * estimate random slopes model
		xi: xtmixed lnscc lnsccel hlnsccel || herd: lnsccel, cov(un) var
   * estimate contextual and random slopes model (two versions)
		xi: xtmixed lnscc lnsccel hlnsccel || herd: lnsccel, cov(un) var
		xi: xtmixed lnscc zlnsccel hlnsccel || herd: lnsccel, cov(un) var
   * extra: group-mean centered random slopes (not the same model!)
		xi: xtmixed lnscc zlnsccel hlnsccel || herd: zlnsccel, cov(un) var

*************************************************************************
* Example 21.7 - Tests for fixed effects
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
* estimate parameters with all predictors (same as Example 21.2)
	xi: xtmixed lnscc shsize heifer i.season sdim || herdid:, var
	testparm _Iseas*

*************************************************************************
* Example 21.8 - Inference for random effects
*************************************************************************
* open the scc40 dataset with one observation per cow
use scc40_2level, clear
   * estimate model without random effects and store results
		xi: xtmixed lnscc shsize heifer i.season sdim, var
			estimates store onelevel
   * estimate models of Example 21.2 and store results
		xi: xtmixed lnscc shsize heifer i.season sdim || herdid:, var
			estimates store randint
   * likelihood-ratio test for random effects
		lrtest onelevel randint, stats
   * manual calculation of P-value
		di chi2tail(1,97.02)/2
   * estimate models of Example 21.4-5 and compute likelihood-ratio tests
		xi: xtmixed lnscc shsize heifer i.season sdim || herdid: sdim, var cov(un)
			estimates store rslopesdim
			lrtest randint rslopesdim, stats
			di chi2tail(2,6.02)/2
		xi: xtmixed lnscc shsize heifer i.season sdim || herdid: heifer, var cov(un)
			estimates store rslopeheifer
			lrtest randint rslopeheifer, stats
			di chi2tail(2,5.79)/2

*************************************************************************
* Example 21.9 - Residuals and diagnostics
*************************************************************************
* open the scc40 dataset with one observation per cow
use scc40_2level, clear
   * reestimate model from Example 21.2
		xi: xtmixed lnscc shsize heifer i.season sdim || herdid:, var
   * cow-level raw residuals and predictions (fixed effects only and both fixed and random effects)
		predict resid_cow, residuals
		predict predf_cow, xb
		predict pred_cow, fitted
		* note: cow-level standardised residuals from xtmixed are just scaled by sigma (not really standardised) 
   * raw herd-level predictions/residuals
		predict resid_herd, reffects level(herdid)
   * herd-level predicted values based on fixed effects
		bysort herdid: egen pred_herd=mean(predf_cow)

   * reanalyse with gllamm to obtain standardised residuals (after ML estimation)
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) adapt
   * standardised cow-level residuals
		gllapred sresid_cow, pearson
   * standardized herd-level residuals
		gllapred res_, ustd 
		rename res_m1 sresid_herd
		drop res_s1
   * herd-level Cook's distance (only for top level)
		gllapred cook_herd, cooksd

   * inspect residuals etc at herd and cow levels
		bysort herd: gen within_herd=_n
		br herdid hsize resid_herd sresid_herd cook_herd pred_herd if within_herd==1
		br herdid cowid resid_cow sresid_cow predf_cow pred_cow

		* Figure 21.4 - Herd level residuals 
		qnorm sresid_herd if within_herd==1, ytitle(Herd-level standardised residual, m(r+2)) ///
			ylabel(,angle(horizontal)) xtitle(inverse normal distribution, m(t+2)) scheme(s1mono) 
			graph save "figs/fig21_res1.gph", replace
			graph export "figs/fig21_res1.wmf", replace
		twoway (scatter sresid_herd pred_herd) if within_herd==1, ///
			ytitle(herd-level standardised residual, m(r+2)) ylabel(,angle(horiz)) ///
			xtitle(herd-level fixed effects prediction, m(t+2)) scheme(s1mono)
			graph save "figs/fig21_res2.gph", replace
			graph export "figs/fig21_res2.wmf", replace
		* combine the two graphs
		graph combine "figs/fig21_res1.gph" "figs/fig21_res2.gph", ///
			ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig21_res_comb.gph", replace
			graph export "figs/fig21_res_comb.wmf", replace
 
	* normal plots and normality tests of standardised residuals at cow level (not in text)
		qnorm sresid_cow
		swilk sresid_cow
   * plots of standardised residuals against predicted values at herd and cow levels (not in text)
		scatter sresid_herd pred_herd if within_herd==1
		scatter sresid_cow pred_cow
		scatter sresid_cow predf_cow

*************************************************************************
* Example 21.10: Box-Cox transformation
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level.dta, clear
	* reconstruct original scc values
		gen scc=exp(lnscc)
   * calculate nobs and meanlnscc
		egen nobs=count(herdid)
		egen meanlnscc=mean(lnscc)
   * set up a file to hold the results for different lambda
		tempname memhold
		postfile memhold lambda lnL pl using temp.dta, replace
		gen y=.
   * loop over lambda values
		foreach lambda of numlist 1 0.5 0.3333 0.25 0 -0.1 -0.25 -0.3333 -0.5 -1 {
			di `lambda'
			replace y=(scc^(`lambda')-1)/(`lambda')
			replace y=lnscc if (`lambda')==0
			xi: xtmixed y shsize heifer i.season sdim, var ml
			scalar lnL=e(ll)
			scalar pl=lnL+n*(`lambda'-1)*meanlnscc
			post memhold (`lambda') (lnL) (pl)
		}
	* stop the process of posting results to a file
		postclose memhold
   * open the file that captured the results
		use temp.dta, clear
		list lambda lnL pl
   * graph for Fig 21.5
		scatter pl lambda, ytitle(profile log-likelihood, m(r+2)) ylabel(,angle(horiz)) ///
			xtitle(lambda, m(t+2)) scheme(s1mono)
		graph save "figs/fig21_bc.gph", replace
		graph export "figs/fig21_bc.wmf", replace

   * analysis of power-transformed data
		gen sccpow=-1/(exp(lnscc))^0.1
		xi: xtmixed sccpow shsize heifer i.season sdim || herdid:, var

*************************************************************************
* Section 21.5.7: Hausman test for Example 21.2
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
   * fixed effects version of model in Example 21.2, fit using the xtreg procedure
		xi: xtreg lnscc shsize heifer i.season sdim , fe i(herdid)
		estimates store fixed
   * random effects model of Example 21.2, fit using the xtreg procedure
		xi: xtreg lnscc shsize heifer i.season sdim, re i(herdid)
		estimates store random 
   * compute Hausman test
		hausman fixed random

*************************************************************************
* Section 21.5.8: Robust standard errors for Example 21.2
*************************************************************************
* open the scc40 dataset with one observation per cow
	use scc40_2level, clear
   * estimation by gllamm (xtmixed does not allow for robust standard errors)
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) adapt
		gllamm, robust
		gllamm, cluster(herdid) /* same result */

*************************************************************************
* Example 21.11: Heteroscedastic standard errors
*************************************************************************
use scc40_2level, clear
   * estimation by gllamm (xtmixed does not allow for heteroscedasticity)
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) adapt
   * standardised cow-level residuals as in Example 21.9
		gllapred sresid_cow, pearson
		bysort heifer: summarize sresid_cow
   * heterogeneous variance model
		gen old=1-heifer
		eq het: heifer old
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) adapt s(het)
   * compute variance and its SE (formula from delta method)
		di exp(.15338837)^2
		di 2*(exp(.15338837)^2)*.02386045
		di exp(.26384704)^2
		di 2*(exp(.26384704)^2)*.02021338
   * random slopes model
		gen cons=1
		eq int: cons
		eq slope: heifer
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) nrf(2) eqs(int slope) adapt
   * heterogeneous variance and random slopes model
		xi: gllamm lnscc shsize heifer i.season sdim, i(herdid) nrf(2) eqs(int slope) adapt s(het)
		di exp(.15135435)^2
		di 2*(exp(.15135435)^2)*.02394571
		di exp(.25904317)^2
		di 2*(exp(.25904317)^2)*.02020538
