* ch22_all.do
* Mixed Models for Discrete Data - all calculations
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch22\data		/* this line is computer dependent */
	capture log close ch22_all
	log using ch22_all, name(ch22_all) replace 

*************************************************************************
* Example 22.1 - Random effects logistic regression model
*************************************************************************
* open the pig respiratory disease dataset
	use pig_adg, clear
   * compute a dichotomous variable for ar greater than 1
		gen ar_g1=ar>1
   * determine the unconditional association between pn and ar_g1
		cc pn ar_g1
   * logistic regression with random farm effects
		xtlogit pn ar_g1, i(farm)
   * same analysis with xtmelogit, in order to display variance
		xtmelogit pn ar_g1 || farm:, var intp(12)
		gllamm pn ar_g1, fam(binom) link(logit) i(farm) trace adapt nip(12)

*************************************************************************
* Example 22.3 - Random Effects Poisson for TB data
*************************************************************************
* open the tb_real dataset
	use tb_real, clear
   * fit a random effects model - normal herd random effects - xtpoisson
		gen logpar=log(par)
		xi:xtpoisson reactors i.type i.sex i.age, off(logpar) i(farm_id) normal
	* same analysis with gllamm
		xi:gllamm reactors i.type sex i.age, fam(pois) link(log) off(logpar) adapt trace ///
			i(farm_id) dots nip(12)
  * same analysis with xtmepoisson, in order to display variance
		xi:xtmepoisson reactors i.type i.sex i.age || farm_id:, off(logpar) var intp(12)
   * compute ICC for young and old (24+ mo) animals
		matrix define est = e(b)
		scalar sigma=exp(est[1,8])
		gen bx=est[1,7]+logpar
		gen younglev1=exp(bx+sigma^2/2)
		gen younglev2=exp(2*bx+2*sigma^2)-exp(2*bx+sigma^2)
		gen young=younglev2/(younglev2+younglev1)
		replace bx=est[1,7]+est[1,6]+logpar
		gen oldlev1=exp(bx+sigma^2/2)
		gen oldlev2=exp(2*bx+2*sigma^2)-exp(2*bx+sigma^2)
		gen old=oldlev2/(oldlev2+oldlev1)
	twoway (connected young logpar, sort msymbol(none) lpattern(longdash)) ///
		(connected old logpar, sort msymbol(none) lpattern(solid)) if logpar>5 & logpar<10, ///
		ytitle(ICC, m(r+2)) ylabel(,angle(horix)) xtitle(log (animal days at risk), m(t+2)) ///
		legend(position(11) ring(0)) scheme(s1mono)

*************************************************************************
* Example 22.4 - Random effects logit and probit regressions for Reunion Island data
*************************************************************************
* open the reu_cfs dataset
	use reu_cfs, clear
   * fit a random effects logistic model - 3 levels (herd, cow, lactation)
		xtmelogit fscr ai heifer || herd: || cow:, var
		* same model fit with gllamm
		gllamm fscr ai heifer, fam(bin) link(logit) adapt i(cow herd) trace dots
   * compute ICCs
		scalar pi=2*acos(0)
		di (.0873799+.2658419)/(.0873799+.2658419+pi^2/3)
		di .0873799/(.0873799+.2658419+pi^2/3)
   * fit a random effects probit using gllamm
		gllamm fscr ai heifer, fam(bin) link(probit) adapt i(cow herd) trace dots
   * compute ICCs
		di (.03259221+.09957098)/(.03259221+.09957098+1)
		di .03259221/(.03259221+.09957098+1)

*************************************************************************
* Example 22.5 - Random effects models for count data
*************************************************************************
* open the tb_real dataset
use tb_real, clear
gen logpar=log(par)
   * Poisson model with log-gamma distributed random effects
		xi:xtpoisson reactors i.type i.sex i.age, off(logpar) i(farm_id)
   * negative binomial model with normal distributed random effects
		* not available in Stata, fit using SAS (proc nlmixed)
   * negative binomial model with beta distributed random effects   
		xi:xtnbreg reactors i.type i.sex i.age, off(logpar) i(farm_id)

*************************************************************************
* Example 22.6 - Random effects proportional odds model
*************************************************************************
* open the tb_real dataset
	use beef_ultra, clear
   * random effects model fit using gllamm
		gllamm grade sex backfat ribeye imfat carc_wt, i(farm) link(ologit) adapt
   * fixed effects model
		xi: ologit grade sex backfat ribeye imfat carc_wt i.farm

*************************************************************************
* Example 22.7 - Checking maximum-likelihood estimation of a GLMM
*************************************************************************
* open the reu_cfs dataset
use reu_cfs, clear
   * fit a random effects logistic model - varying # of quadrature points
	set more off
	xtmelogit fscr ai heifer || herd: || cow:, var intpoints(1 1)
	xtmelogit fscr ai heifer || herd: || cow:, var intpoints(3 3)
	xtmelogit fscr ai heifer || herd: || cow:, var intpoints(7 7)
	xtmelogit fscr ai heifer || herd: || cow:, var
	xtmelogit fscr ai heifer || herd: || cow:, var intpoints(12 12)
	*gllamm fscr ai heifer , fam(bin) link(logit) adapt trace i( cow herd) dots nip(6)
	*gllamm fscr ai heifer , fam(bin) link(logit) adapt trace i( cow herd) dots nip(8)
	*gllamm fscr ai heifer , fam(bin) link(logit) adapt trace i( cow herd) dots nip(10)
	*gllamm fscr  aiheifer , fam(bin) link(logit) adapt trace i( cow herd) dots nip(12)

*************************************************************************
* Example 22.8 - Quasi-likelihood estimation of a GLMM
*************************************************************************
* all analyses carried out in MLwiN (no Stata equivalent)

*************************************************************************
* Example 22.9 - Statistical inference in a GLMM
*************************************************************************
* open the reu_cfs dataset
use reu_cfs, clear
   * fit reduced random effects logistic models
	xtmelogit fscr ai heifer || herd:, var
	xtmelogit fscr ai heifer || cow:, var
	*gllamm fscr ai heifer, fam(bin) link(logit) adapt trace i(herd) dots
	*gllamm fscr ai heifer, fam(bin) link(logit) adapt trace i(cow) dots

*************************************************************************
* Example 22.10
*************************************************************************
use pig_adg, clear
   * compute a dichotomous variable for ar greater than 1
	gen ar_g1=ar>1
   * logistic regression with random farm effects
   * code from Skrondal & Rabe-Hesketh (2009)
		gllamm pn ar_g1, fam(binom) link(logit) i(farm) trace adapt nip(12)
		gen zero1=0
		gllapred p1, mu us(zero) 
		gllapred p2, mu marg
		predict u, u
		gllapred p1u, mu us(um)
		gllapred p3, mu
   * attempt to compute from xtmelogit
		capture drop p1 p2 p3 p1u
		xtmelogit pn ar_g1 || farm:, var intp(12)
		predict p1, mu fixedonly
		predict xb, xb 
		gen p2=invlogit(xb/sqrt(1+0.346*0.877))
		predict u, reffects
		gen p1u=invlogit(xb+u)
		predict p3, mu
		* note that p3=p1u (which it should not be!)

*************************************************************************
* Example 22.11 - Residuals from a 3-level GLMM (xtmelogit)
*************************************************************************
* open the reu_cfs dataset
	use reu_cfs, clear
   * reestimate model from Example 21.4
		xtmelogit fscr ai heifer || herd: || cow:, var
   * cow-level raw residuals and predictions (fixed effects only and both fixed and random effects)
		predict xb, xb
		predict ref_cow, ref level(cow)
		predict rese_cow, rese level(cow)
		gen stdres_cow=ref_cow/rese_cow
		predict ref_herd, ref level(herd)
		predict rese_herd, rese level(herd)
		gen stdres_herd=ref_herd/rese_herd
	* normal plots of standardised residuals at cow and herd levels
		bysort herd: gen within_herd=_n
		bysort herd cow: gen within_cow=_n
	* Normal probability plot of cow level residuals
		qnorm stdres_cow if within_cow==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(cow level std. residuals,m(r+2)) xtitle(,m(t+2))
	* Normal probability plot of herd level residuals
		qnorm stdres_herd if within_herd==1,scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(herd level std. residuals,m(r+2)) xtitle(,m(t+2))  
			graph save "figs/fig22_res1.gph", replace
	* compute herd-level predicted values (linear predictor) based on fixed effects
		bysort herd: egen pred_herd=mean(xb)
	* plots of standardised residuals against predicted values at herd level
		scatter stdres_herd pred_herd if within_herd==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(herd level std. residuals,m(r+2)) xtitle(herd level linear prediction,m(t+2)) 
			graph save "figs/fig22_res2.gph", replace

*************************************************************************
* Example 22.11 - Residuals from a 3-level GLMM (gllamm)
*************************************************************************
* open the reu_cfs dataset
	set more off
	use reu_cfs, clear
	* reestimate model from Example 21.4
	* reanalyse with gllamm to obtain diagnostic standardised residuals
		gllamm fscr ai heifer, fam(bin) link(logit) adapt i(cow herd) trace dots
   * standardised residuals at cow and herd levels
		gllapred res_, ustd
		drop res_s1 res_s2
		gllapred xb, xb
   * normal plots of standardised residuals at cow and herd levels
		bysort herd: gen within_herd=_n
		bysort herd cow: gen within_cow=_n
	* Fig 22.2 Normal probability plot of cow level residuals
		qnorm res_m1 if within_cow==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(cow level std. residuals,m(r+2)) xtitle(,m(t+2))
	* Normal probability plot of herd level residuals
		qnorm res_m2 if within_herd==1,scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(herd level std. residuals,m(r+2)) xtitle(,m(t+2))  
		graph save "figs/fig22_res1.gph", replace
   * compute herd-level predicted values (linear predictor) based on fixed effects
		bysort herd: egen pred_herd=mean(xb)
	* plots of standardised residuals against predicted values at herd level
		scatter res_m2 pred_herd if within_herd==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(herd level std. residuals,m(r+2)) xtitle(herd level linear prediction,m(t+2)) 
		graph save "figs/fig22_res2.gph", replace
	* Fig 22.3 combine the two graphs
		graph combine figs/fig22_res1.gph figs/fig22_res2.gph , ysize(3) xsize(8) scheme(s1mono)
		graph save "figs/fig22_res_comb.gph", replace
		graph export "figs/fig22_res_comb.wmf", replace

*************************************************************************
* Example 22.12 - Aggregation of lowest level for pig seroconversion data
*************************************************************************
	* Analysis carried out in MLwiN (no Stata equivalent)

*************************************************************************
* Example 22.13 - Analyses for pig pneumonia data
*************************************************************************
* open the pig respiratory disease dataset
	use pig_adg, clear
   * compute a dichotomous varaible for ar greater than 1
		gen ar_g1=ar>1
   * ordinary logistic regression
		logit pn ar_g1
   * robust variance (clustered at farms)
		logit pn ar_g1, vce (cluster farm)
   * fixed farm effects
		xi: logit pn ar_g1 i.farm
   * stratification by M-H method
		cs pn ar_g1, by(farm) or
   * maximum likelihood estimation
		xtmelogit pn ar_g1 || farm:, var intp(12)
   * GEE with exchangeable correlation structure
		xtgee pn ar_g1, fam(bin) i(farm) robust 
   * Bayesian analysis carried out in MLwiN
