* ch23_all.do - command for some examples and figures in Chapter 23
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch23\data		/* this line is computer dependent */
	capture log close ch23_all
	log using ch23_all, name(ch23_all) replace 

*************************************************************************
* Example 23.1 Graphs and descriptive statistics for fish trial data
*************************************************************************
use fish_trial, clear
   * table of means
	table jaw sample, contents(mean lnwt sd lnwt count lnwt) by(vaccine)
   * profile plot
		twoway  (connected lnwt day if fish==45, msymbol(none) lcolor(black) lpattern(longdash)) ///
       (connected lnwt day if fish==46, msymbol(none) lcolor(black)) ///
       (connected lnwt day if fish==47, msymbol(none) lcolor(black)) ///
       (connected lnwt day if fish==48, msymbol(none) lcolor(black)) /// 
       (connected lnwt day if fish==49, msymbol(none) lcolor(black) lpattern(longdash)) ///
       (connected lnwt day if fish==50, msymbol(none) lcolor(black)) ///
       (connected lnwt day if fish==51, msymbol(none) lcolor(black)) ///
       (connected lnwt day if fish==52, msymbol(none) lcolor(black)) /// 
       (connected lnwt day if fish==53, msymbol(none) lcolor(black)) ///
       (connected lnwt day if fish==54, msymbol(none) lcolor(black) lpattern(longdash)) ///
       (connected lnwt day if fish==55, msymbol(none) lcolor(black) lpattern(longdash)) ///
       (connected lnwt day if fish==56, msymbol(none) lcolor(black)) /// 
       (connected lnwt day if fish==57, msymbol(none) lcolor(black) lpattern(longdash)) ///
       (connected lnwt day if fish==58, msymbol(none) lcolor(black)) /// 
       (connected lnwt day if fish==59, msymbol(none) lcolor(black)), ///
	ytitle(natural log of weight, m(r+2)) xtitle(days since tagging, m(t+2)) ///
	plotregion(style(none)) legend(off) ylabel(, angle(horizontal) format(%4.0f)) ///
	scheme(s1mono)
	graph save "figs/fig23_profplot.gph", replace
	graph export "figs/fig23_profplot.wmf", replace
		/* note - the above command assumes that there is a "figs" directory under your working directory */
   * mean plot for jaw
		bysort sample jaw: gen within=_n
		bysort sample jaw: egen meanlnwt=mean(lnwt)
		twoway (connected meanlnwt day if within==1 & jaw==0, msymbol(none) lcolor(black)) ///
			(connected meanlnwt day if within==1 & jaw==1, msymbol(none) lcolor(black) lpattern(longdash)) ///
			, ytitle(mean natural log of weight, m(r+2)) xtitle(days since tagging, m(t+2)) legend(off) ///
			plotregion(style(none)) ylabel(, angle(horizontal) format(%4.0f)) ///
			scheme(s1mono)
		graph save "figs/fig23_meanplot.gph", replace
		graph export "figs/fig23_meanplot.wmf", replace
		/* note - the above command assumes that there is a "figs" directory under your working directory */
	* combine the two graphs
		graph combine "figs/fig23_profplot.gph" "figs/fig23_meanplot.gph" , ysize(4) xsize(8) scheme(s1mono)
		graph export "C:\ver2\ch19\figs\fig23_plot_comb.wmf", replace

	* correlations
		preserve
			keep sample fish lnwt
			reshape wide lnwt, i(fish) j(sample)
			sum lnwt1-lnwt5, d
			corr lnwt1-lnwt5, cov
			corr lnwt1-lnwt5
		restore

*************************************************************************
* Example 23.2 Graphs and descriptive statistics for somatic cell count data
*************************************************************************
use scc40.dta, clear
rename t_lnscc lnscc
   * profile plot of 21 cows from herd 3
		twoway  (connected lnscc test if cowid==141, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==142, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==143, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==144, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==145, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==146, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==147, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==148, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==149, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==150, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==151, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==152, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==153, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==154, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==155, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==156, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==157, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==158, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==159, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==160, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			(connected lnscc test if cowid==161, msize(small) msymbol(circle) mcolor(black) lcolor(black)) ///
			, ytitle(log somatic cell count, m(r+2)) xtitle(month of lactation, m(t+2)) legend(off) ///
			plotregion(style(none)) ylabel(, angle(horizontal) format(%4.0f)) ///
			scheme(s1mono)
		graph save "figs/fig23_profscc.gph", replace
		graph export "figs/fig23_profscc.wmf", replace

*************************************************************************
* Example 23.3 Univariate methods for fish trial data
*************************************************************************
use fish_trial, clear
   * analysis at separate time points
		anova lnwt vaccine jaw vaccine#jaw if sample==1
		anova lnwt vaccine jaw vaccine#jaw if sample==2
		anova lnwt vaccine jaw vaccine#jaw if sample==3
		anova lnwt vaccine jaw vaccine#jaw if sample==4
		anova lnwt vaccine jaw vaccine#jaw if sample==5
   * summary statistic: gain
		preserve
			keep sample fish lnwt vaccine jaw
			reshape wide lnwt, i(fish) j(sample)
			gen gain=lnwt5-lnwt1
			anova gain vaccine jaw vaccine#jaw
		restore
   * summary statistic: slope for samples 3-5
		gen slope=.
		* loop through the fish and compute slope for samples 3-5
		set more off
		forvalues xx=1/400 {
			* di `xx'
			quietly reg lnwt day if fish==`xx' & sample>=3
			replace slope=_b[day] if fish==`xx'
			}
		* identify 1 observation per fish
			gen rec1=.
			bysort fish: replace rec1=(_n==1)
		* ANOVA of slopes
			anova slope vaccine jaw vaccine#jaw if rec1==1

*************************************************************************
* Example 23.4 ANOVA and MANOVA analysis of fish trial data
*************************************************************************
use fish_trial, clear
   * random intercept model analysed by ANOVA method
		anova lnwt vaccine jaw vaccine*jaw / fish|vaccine*jaw sample sample*vaccine sample*jaw sample*vaccine*jaw
   * random intercept model analysed by maximum likelihood method
   * note: xi does not work with three-factor interactions (and xi3 does not work with xtmixed)
	gen vaccjaw=vaccine+10*jaw
	xi: xtmixed lnwt i.vaccjaw*i.sample || fish:, var
   * repeated measures ANOVA
	anova lnwt vaccine jaw vaccine*jaw / fish|vaccine*jaw sample sample*vaccine sample*jaw sample*vaccine*jaw, repeated(sample)
   * MANOVA 
	preserve
		keep sample fish lnwt vaccine jaw
		reshape wide lnwt, i(fish) j(sample)
		gen gain=lnwt5-lnwt1
		manova lnwt1 lnwt2 lnwt3 lnwt4 lnwt5 = vaccine jaw vaccine*jaw
			* note: calculation of tests involving time carried out in SAS, proc glm
		* calculation of estimated variance-covariance matrix
		matrix define e = e(E)
		di e[1,1]/392
		di e[2,2]/392
		di e[3,3]/392
		di e[4,4]/392
		di e[5,5]/392
		di e[1,2]/sqrt(e[1,1]*e[2,2])
		di e[1,3]/sqrt(e[1,1]*e[3,3])
		di e[1,4]/sqrt(e[1,1]*e[4,4])
		di e[1,5]/sqrt(e[1,1]*e[5,5])
		di e[2,3]/sqrt(e[2,2]*e[3,3])
		di e[2,4]/sqrt(e[2,2]*e[4,4])
		di e[2,5]/sqrt(e[2,2]*e[5,5])
		di e[3,4]/sqrt(e[3,3]*e[4,4])
		di e[3,5]/sqrt(e[3,3]*e[5,5])
		di e[4,5]/sqrt(e[4,4]*e[5,5])
	restore

*************************************************************************
* Example 23.5 Linear mixed models with correlation structure for somatic cell count data
*************************************************************************
	* analysis carried out in SAS, proc mixed
	* capabilities for autocrrelated errors added to Stata in Version 11


*************************************************************************
* Example 23.6 Trend model for somatic cell count data
*************************************************************************
use scc40b.dta, clear
	* Note - this is a slightly different version of scc40 than is described in the text
		* some changes in variable names and some variables rescaled
	* random intercept model
		xi: xtmixed lnscc i.season shsize heifer sdim || herdid: || cowid:, var
	* random slope model with linear effect of sdim at cow level only
		xi: xtmixed lnscc i.season shsize heifer sdim || herdid: || cowid: sdim, cov(uns) var
	* random slope model with linear effect of sdim at cow and herd level
		xi: xtmixed lnscc i.season shsize heifer sdim || herdid: sdim, cov(uns) || cowid: sdim, cov(uns) var


*************************************************************************
* Example 23.7 Generalised linear mixed models for somatic cell count data
*************************************************************************
use scc40b.dta, clear
   * generate binary outcome
		gen highscc=lnscc>ln(200)
		replace highscc=. if lnscc==. /* no missing values */
   * descriptive statistics for highscc
		table highscc
		summarize highscc
		bysort cowid: egen within_cow_sd=sd(highscc)
		bysort cowid: gen within_cow=_n
		table within_cow_sd highscc if within_cow==1, row col
   * random intercept model
		xi: xtmelogit highscc i.season shsize heifer sdim || herdid: || cowid:, var
   * transitional model
		gen laghighscc=highscc[_n-1]
		gen lagtest=test[_n-1]
		bysort cowid: gen within_cow=_n
		replace laghighscc=. if within_cow==1 & lagtest~=test-1
		xi: xtmelogit highscc i.season shsize heifer sdim laghighscc || herdid: || cowid:, var
   * rerun random intercept model on reduced dataset to see if that affects the variance
		xi: xtmelogit highscc i.season shsize heifer sdim || herdid: || cowid: if laghighscc~=., var
   * trend model
		xi: xtmelogit highscc i.season shsize heifer sdim || herdid: || cowid: sdim, cov(uns) var


*************************************************************************
* Example 23.8 - GEE analysis for pig-pneumonia data
*************************************************************************
* open the pig dataset
use pig_adg, clear
   * compute a dichotomous varible for ar greater than 1
		gen ar_g1=ar>1
   * run GEE estimation with compound symmetry working correlation structure
		xtgee pn ar_g1, i(farm) fam(bin) link(logit) robust
		xtcorr , compact


*************************************************************************
* Example 23.9 - GEE - somatic cell count data
*************************************************************************
* open the SCC dataset
use scc_40b, clear
	* run GEE estimation with compound symmetry working correlation structure and QIC
		xi: qic lnscc shsize heifer i.season sdim, i(cowid) fam(gaus) link(ident) corr(exch) robust
		xtcorr, compact
	* run GEE estimation with autoregressive (ar1) working correlation structure
		xi: qic lnscc shsize heifer i.season sdim, i(cowid) t(test) fam(gaus) link(ident) corr(ar1) robust force
		xtcorr, compact
	* run GEE estimation with unstructured working correlation structure
		xi: qic lnscc shsize heifer i.season sdim, i(cowid) t(test) fam(gaus) link(ident) corr(uns) robust force
		xtcorr, compact
	* run GEE estimation with stationary working correlation structure
	* still does not work, for 9 or 10
		xi: qic lnscc shsize heifer i.season sdim, i(cowid) t(test) fam(gaus) link(ident) corr(sta 9) robust force
		xtcorr, compact
	* run GEE estimation with compound symmetry working correlation structure at herd level and QIC
		xi: qic lnscc shsize heifer i.season sdim, i(herdid) fam(gaus) link(ident) corr(exch) robust
		xtcorr, compact

*************************************************************************
* Example 23.10: GEE and ALR estimation for somatic cell count data
*************************************************************************
* open the SCC dataset
	use scc_40b, clear
	* generate binary outcome
	gen highscc=lnscc>ln(200)
	replace highscc=. if lnscc==. /* no missing values */
	* ordinary logistic regression
		xi: logit highscc shsize heifer i.season sdim
	* run GEE estimation with autoregressive (ar1) working correlation structure
		xi: xtgee highscc shsize heifer i.season sdim, i(cowid) t(test) fam(bin) link(logit) corr(ar1) robust force
	xtcorr, compact
	* run GEE estimation with autoregressive (ar1) working correlation structure
		xi: xtgee highscc shsize heifer i.season sdim i.herdid, i(cowid) t(test) fam(bin) link(logit) corr(ar1) robust force
		xtcorr, compact
   * run GEE estimation with exchangeable herd-level working correlation structure
		set matsize 800
		xi: xtgee highscc shsize heifer i.season sdim, i(herdid) fam(bin) link(logit) corr(exch) robust force
		xtcorr, compact
   * ALR analysis run in SAS proc genmod
