* ch27_all.do
* Concepts of Infectious Disease Epidemiology - all calculations
*
	clear all
	version 10
	set more off 
	set memory 20m
	cd c:\ver2\ch27\data		/* this line is computer dependent */
	capture log close ch27_all
	log using ch27_all.log, text name(ch27_all) replace 
		* list of cages used in analyses
		*	77 78 79 80 81 82 83 84 86 

*************************************************************************
* Ex 27.1 Computing doubling times
*************************************************************************

use isa_day.dta

* set up a file ("isa_dbl_time_s28.dta") to hold the results from the analyses
	tempname memhold
	postfile `memhold' cage beta dblt R0 using "isa_dbl_time_s28.dta", replace

	foreach cg of numlist 77/84 86 {
		di `cg'
		preserve
		keep if cage==`cg'
	* select the exponential phase
		keep if stage==1
	* log transform the daily mortality
		gen ln_md=log(mort_d)
	* fit a log-linear regression
		quietly reg ln_md dt
			predict xb
			gen yhat=exp(xb)
	* compute the doubling time
		matrix B=e(b)
		scalar beta=B[1,1]
		scalar dblt=log(2)/beta
		scalar R0=beta*21 +1
		scalar cage_id=`cg'
	* generate graphs with exponential curve overlaying the spikeplot
		twoway (spike mort_d dt) (line yhat dt), title("Cage `cg'") ///
			ylabel(#2, angle(horizontal)) yti("Daily mortalities") xti("Date") xlabel(minmax) ///
			scheme(s1mono) saving(dblt_`cg'.gph, replace) legend(off)
	* post results to file 
		post `memhold' (cage) (beta) (dblt) (R0)
	restore
		}
	* stop the process of posting results to a file
	postclose `memhold'
* generate some summary statistics
	use isa_dbl_time_s28.dta, clear
	bysort cage: sum beta dblt R0

* combine the graphs from 4 cages
	graph combine dblt_81.gph dblt_82.gph dblt_83.gph dblt_84.gph , ///
		imargin(r+5) scheme(s1mono) saving(dblt, replace)
	graph save "figs/dblt.gph", replace


*************************************************************************
* Fig 27.6 Generating a generic survival curve (exponential)
*************************************************************************

twoway (function exp(-0.01*x), range(0 500)) ///
  (function exp(-1)*(2-.01*x), range(40 160) ///
       lcolor(black) lwidth(medthick)), ///
	yti("Prop. susceptible (s)", m(r+2)) ylab(0 .5 1, angle(horizontal)) ///
	xti("Age", m(t+2)) scheme(s1mono) legend(off)
graph save "figs/s_vs_t.gph", replace
graph export "figs/s_vs_t.wmf", replace


*************************************************************************
* Ex 27.2 Neg. bin. regression of ISA weekly data
*************************************************************************

use isa_wk.dta, clear
	sort cage week
	keep if week>=0
	* display some of the data
		list cage week par C S E Ia Ib R N if cage==81, clean noobs

* compute offsets so they are based upon week before E
	by cage: gen offset=log(S[_n-1]*I[_n-1]/N[_n-1])

* set up a file to hold the results from the analyses
	tempname memhold
	postfile `memhold' cg_id cnvg beta R0 ll using "isa_neg_bin.dta", replace

* Loop through the cages individually
	foreach cg of numlist 77/84 86 { 
 	   di _newline "Cage `cg'"
	   * since this is a constant-only model it should converge quickly...
      * there should be no need for the iterate() option
		nbreg E if cage==`cg', offset(offset) 
			* compute R0
				matrix B=e(b)
				scalar beta=B[1,1]
				scalar R0=exp(beta)*2
			post `memhold' (`cg') (e(converged)) (beta) (R0) (e(ll))
	}
   * stop the process of posting results to a file
	postclose `memhold'

* list the results
	use isa_neg_bin.dta, clear
	list, clean noobs

* random effects negative binomial regression (all cages together)
	* model fit using MLwiN to allow for normally distributed random effects

* Ex 27.3 sensitivity analysis (varying proportion of infected fish dying)
	* code not shown (but similar to above with modifications to the data)


*************************************************************************
* Fig 27.7 Spikeplot for cage 83
*************************************************************************

use isa_day.dta, clear
keep if cage==83
* generate the spikeplot
	twoway spike mort_d dt, scheme(s1mono) xlab(13666 13696 13727 13758) ///
		ytitle("daily mortality", m(r+2)) ylabel(#5, angle(horizontal)) ///
		xtitle("date", m(t+2)) plotregion(style(none)) ///
		ti("Cage 83") 
	graph save "figs/cage83.gph", replace
	graph export "figs/cage83.wmf", replace


*************************************************************************
* Ex 27.4 Generating the graph of observed and expected for diff eq section
*************************************************************************
use "solutions2.dta", clear 	/* data set not provided on web site */
* generate the graph
	twoway (scatter obs time) (line a1pc_2 time, lpattern(shortdash)lcolor(black)) ///
		(line a1pc_5 time, lpattern(shortdash) lcolor(black)) ///
		(line a_5pc_3 time, lpattern(longdash)lcolor(black)) ///
		(line a_2pc_3 time, lpattern(longdash)lcolor(black)) ///
		(line best time, lpattern(solid)lcolor(black)) , ///
		scheme(s1mono) legend(off) xti("Days") yti("Mortalities", margin(r+5)) ///
		ylab(0(500)2500, angle(horizontal))
	graph save "C:\ver2\ch27\figs\obs_exp.gph", replace
	graph export "C:\ver2\ch27\figs\obs_exp.wmf", replace
	/* options not used - graph manually edited
		text(1500 27 "cp=0.5") 	text(950 100 "cp=0.2") text(800 75 "a=0.5") ///
		text(325 65 "a=0.2") text(1400 85 "best fit") ///
	*/
