* ch28_all.do
* Meta-analysis
	clear all
	version 10
	set more off 
	set memory 20m
	set matsize 200
	cd c:\ver2\ch28\data		/* this line is computer dependent */
	capture log close ch28_all
	log using ch28_all.log, text name(ch28_all) replace 

*************************************************************************
*Ex 28.3 - 28.4  Fixed and random effects meta-analyses, forest plot
*************************************************************************
	* rBST - milk dataset
		use bst_milk.dta, clear
		meta md md_se
		metan md md_se, fixed
		metan md md_se, random 
	* rBST - mastitis dataset
		use bst_mast.dta, clear
		meta rr cilow cihigh, ci eform
* Forest plot - mastitis data
	use bst_mast.dta, clear
	metan rr_ln rr_ln_se, eform xlabel(.25, .5, 1, 2, 4, 10) force ///
		scheme(s1mono) xtitle("RR for clinical mastitis", size(small))
		graph save "figs/fig28_forrest.gph", replace
		graph export "figs/fig28_forrest.wmf", replace


*************************************************************************
* Ex 28.5 Stratified analysis - milk production data
*************************************************************************
	use bst_milk.dta, clear
	metan md md_se, by(parity) fixed 

*************************************************************************
* Section 28.7.3 Heterogeneity - Graphical assessment and meta-regression
*************************************************************************
* Fig 28.3 Galbraith Plot
	use bst_milk.dta, clear
	metan md md_se, fixed
		scalar es=r(ES)
		gen y=md/md_se
		gen x=1/md_se
		gen l1=x*es
		gen l2=l1+2
		gen l3=l1-2
	twoway (scatter y x, mlabel(group)) (line l1 x, lcolor(black) lwidth(medium)) ///
		(line l2 x, lcolor(black) lwidth(medium) ) ///
		(line l3 x, lcolor(black) lwidth(medium) ), scheme(s1mono) ///
		legend(off) yti("Z = T / SE", m(r+2)) xti("1/SE", m(r+2)) ylabel(, angle(horizontal)) ///
		xlabel(0.5 1 1.5 2 2.5)  
		graph save "figs/fig28_glbrth.gph", replace
		graph export "figs/fig28_glbrth.wmf", replace
* Fig 28.4 Scatter plot assessment of heterogeneity
	use bst_mast.dta, clear
	metan rr_ln rr_ln_se, eform
	format dose_day %6.0f 
	twoway (scatter rr_ln dose_day [aweight=_WT], ///
		msymbol(circle_hollow) msize(small) mcolor(black) mlwidth(medium)) ///
		(lfit rr_ln dose_day, lwidth(medium) lcolor(black)) (scatter rr_ln dose_day, msize(vtiny) mlabel(group)) ///
		 , scheme(s1mono) legend(off) xti("daily dose (mg/day)", m(t+2)) yti("ln RR", m(r+2)) ///
		ylabel( , angle(horizontal))
		graph save "figs/fig28_het_scatter.gph", replace
		graph export "figs/fig28_het_scatter.wmf", replace

*************************************************************************
* Ex 28.6 Meta-regression
*************************************************************************
	use bst_milk.dta, clear
	xi: metareg md i.parity i.company dose_day, wsse(md_se) bsest(reml)
		test _Iparity_2 _Iparity_3
		test  _Icompany_2 _Icompany_3 _Icompany_4

*************************************************************************
* Fig 28.5 L'Abbe plot
*************************************************************************
	use bst_mast.dta, clear
	metan rr_ln rr_ln_se, eform 
		scalar es=r(ES)
	gen r_tx=tx_pos/(tx_pos+tx_neg)
	gen r_ctrl=ctrl_pos/(ctrl_pos+ctrl_neg)
	gen l1=r_ctrl*es
	twoway (scatter r_tx r_ctrl [aweight=ncows], msymb(Oh) msize(small) mcolor(black) mlwidth(medium)) ///
		(line l1 r_ctrl, lwidth(medium) lcolor(black)) ///
		(scatter r_tx r_ctrl , msize(vtiny) mlabel(group)), ///
		legend(off) scheme(s1mono) xti("risk in control group", m(t+2)) xlab(0 .2 .4 .6) ///
		yti("risk in treated group", m(r+2)) ylab(0 .2 .4 .6 .8, angle(horizontal))
		graph save "figs/fig28_labbe.gph", replace
		graph export "figs/fig28_labbe.wmf", replace

*************************************************************************
* Section 28.8 Publication bias - parasite data
*************************************************************************
	use meta_parasite.dta, clear
	metan md md_se
	* Fig 28.7 funnel plot and Begg's and Egger's tests
		metabias md md_se if mlkmeas==2, begg
		metabias md md_se if mlkmeas==2, egger
		metafunnel md md_se if mlkmeas==2, ytitle("SE of treatment effect", m(r+2)) ///
			ylabel(, angle(horizontal)) xtitle(, m(t+2)) plotregion(style(none)) ///
			scheme(s1mono)
		graph save "figs/fig28_funnel.gph", replace
		graph export "figs/fig28_funnel.wmf", replace
	* Ex 28.8 perform Duval and Tweedie's "Trim and Fill" procedure
		metatrim md md_se if mlkmeas==2, re funnel t2("Anthelmintic Treatment") ///
		save(trim_fill, replace)
			use trim_fill, clear
		metafunnel filled fillse , ytitle("SE of treatment effect", m(r+2)) ///
			ylabel(, angle(horizontal)) xtitle(treatment effect, m(t+2)) plotregion(style(none)) ///
			scheme(s1mono)
				* add boxes to filled points using the Stata graphics editor
			graph save "figs/fig28_trimfill.gph", replace
			graph export "figs/fig28_trimfill.wmf", replace

*************************************************************************
* Ex 28.9 Influential studies
*************************************************************************
	use bst_mast.dta, clear
	metaninf rr_ln rr_ln_se, random id(group) t2("Clin Mast - Risk") print b1("lnRR")
		graph save "figs/fig28_infl.gph", replace
		graph export "figs/fig28_infl.wmf", replace

